Package: HTDV
Type: Package
Title: Hypothesis Testing for Dependent Variables with Unbalanced Data
Version: 0.2.0
Authors@R: person(given = "Jose Mauricio", family = "Gomez Julian",
    email = "isadorenabi@pm.me", role = c("aut", "cre"),
    comment = c(ORCID = "0009-0000-2412-3150"))
Description: Implements hierarchical Bayesian inference, robust frequentist
    inference, and distribution-free inference for dependent and unbalanced
    data under strong-mixing conditions. Supports triangular-array,
    weighted-sum and mixingale convergence regimes with Whittle and composite
    likelihoods, heteroskedasticity-and-autocorrelation-consistent variance
    estimation, block bootstrap with automatic block length, fixed-bandwidth
    HAR inference, adaptive conformal prediction, Bayesian decision under
    Region of Practical Equivalence, bridge-sampling Bayes factors, and
    predictive comparison via the Widely Applicable Information Criterion and
    leave-future-out cross-validation. Methods follow Andrews (1991)
    <doi:10.2307/2938229>, Kiefer and Vogelsang (2005)
    <doi:10.1017/S0266466605050565>, Patton, Politis and White (2009)
    <doi:10.1080/07474930802459016>, Vehtari, Gelman and Gabry (2017)
    <doi:10.1007/s11222-016-9696-4>, Kruschke (2018)
    <doi:10.1177/2515245918771304>, and Gibbs and Candes (2021)
    <doi:10.48550/arXiv.2106.00170>.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.1.0)
Imports: rstan (>= 2.26.0), stats, parallel,
Suggests: bayesplot, bridgesampling, knitr, loo, posterior, readxl,
        rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
RoxygenNote: 7.3.3
URL: https://github.com/IsadoreNabi/HTDV
BugReports: https://github.com/IsadoreNabi/HTDV/issues
NeedsCompilation: no
Packaged: 2026-04-29 15:05:39 UTC; ROG
Author: Jose Mauricio Gomez Julian [aut, cre] (ORCID:
    <https://orcid.org/0009-0000-2412-3150>)
Maintainer: Jose Mauricio Gomez Julian <isadorenabi@pm.me>
Repository: CRAN
Date/Publication: 2026-05-02 09:50:14 UTC
Built: R 4.6.0; ; 2026-05-02 12:00:58 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,htdv_fit)
S3method(print,htdv_fit)
S3method(summary,htdv_fit)
export(htdv_bf)
export(htdv_boot)
export(htdv_composite)
export(htdv_conformal)
export(htdv_diagnostics)
export(htdv_envelope)
export(htdv_equivalence_constants)
export(htdv_fit)
export(htdv_fixedb)
export(htdv_lrv)
export(htdv_mpc)
export(htdv_ppc)
export(htdv_rope)
export(htdv_simstudy)
export(htdv_simstudy_summary)
export(htdv_simstudy_warnings)
export(htdv_stack)
export(htdv_summary)
export(htdv_tac)
export(htdv_waic_lfo)
export(htdv_whittle)
export(htdv_wsc)

# HTDV

Hypothesis Testing for Dependent Variables with Unbalanced Data.

`HTDV` provides a unified R toolkit for inference on dependent,
unbalanced data under strong-mixing conditions, combining hierarchical
Bayesian estimation via Hamiltonian Monte Carlo with frequentist and
distribution-free robustness anchors (fixed-b HAR, block bootstrap,
adaptive conformal).

## Validation card

The framework is shipped with two pre-registered validation studies,
both reproducible end-to-end and with their summary tables exposed as
package datasets. See `vignette("HTDV-validation")`.

- **Factorial Monte Carlo** (`htdv_sim_summary`). 1024-cell design
  crossing sample size, AR(1) coefficient, innovation tail, imbalance
  ratio and location shift; 500 replications per cell × 3 inferential
  layers; 31 hours of wall-clock on 16 cores. The Bayesian envelope
  holds nominal size (mean 0.0556, sd 0.013) and nominal coverage (mean
  0.944) across the entire grid; HAR and bootstrap inflate to
  empirical size 0.60 and coverage 0.29 in the worst corners under
  strong persistence. The asymptotic gap that motivates the framework
  is *visible in the data*.
- **External benchmarks** (`htdv_empirical_benchmarks`). Three public
  datasets compared against published references:
  - FRED-MD post-1984 CPI inflation against Stock and Watson (2007).
  - Shiller log-CAPE against Campbell and Shiller (1998).
  - US-Canada 10-year yield differential against the iid Welch baseline.

  All three layers reproduce all three references with `agreement` in
  every case. The 95% interval widths scale monotonically with the
  series persistence: at $\widehat\phi\approx 0.45$ Bayes is 0.81× HAR;
  at $\widehat\phi\approx 0.97$ it is 2.80× HAR; at near-unit-root
  ($\widehat\phi\approx 0.99$) it is 15.0× HAR. The framework's value
  is the *visibility* of this gradient.

```{r, eval = FALSE}
library(HTDV)
data(htdv_sim_summary)         # simulation summary, 3069 rows
data(htdv_empirical_benchmarks) # three-dataset external validation
vignette("HTDV-validation")    # full narrative
```

## Installation

```r
remotes::install_github("IsadoreNabi/HTDV")
```

`rstan` is required. Optional backends: `bridgesampling` (Bayes factors),
`loo` (WAIC / PSIS-LOO), `posterior` (draws utilities), `bayesplot`
(visualization), `readxl` (vignette).

## Core API

| Function | Purpose |
|----------|---------|
| `htdv_fit()` | Hierarchical Bayesian HMC fit. |
| `htdv_envelope()` | Berger-robust envelope across models. |
| `htdv_lrv()` | HAC long-run variance (Andrews bandwidth). |
| `htdv_fixedb()` | Fixed-bandwidth HAR Wald test. |
| `htdv_boot()` | Block bootstrap with automatic block length. |
| `htdv_conformal()` | Adaptive conformal inference. |
| `htdv_rope()` | ROPE-based posterior decision. |
| `htdv_bf()` | Bridge-sampling Bayes factor. |
| `htdv_waic_lfo()` | WAIC and leave-future-out CV. |
| `htdv_stack()` | Predictive stacking. |
| `htdv_diagnostics()` | MCMC diagnostics. |
| `htdv_ppc()` | Posterior-predictive checks on dependence statistics. |
| `htdv_equivalence_constants()` | Explicit TAC/WSC/MPC constants. |
| `htdv_simstudy()` | Factorial Monte Carlo study (Section 12-bis). |
| `htdv_simstudy_summary()` | Aggregate per-cell results. |
| `htdv_simstudy_warnings()` | Flag cells in the limit-of-identification zone. |

See `vignette("HTDV-intro")` for a walkthrough,
`vignette("HTDV-validation")` for the full validation report.

## Citation

Please cite both the package and the companion paper. Run
`citation("HTDV")` for the current BibTeX entries.

## License

MIT.

#' Factorial Monte Carlo Simulation Study for Dependent-Unbalanced Data
#'
#' Runs the pre-registered factorial Monte Carlo study of Section 12-bis of
#' the companion paper. The study crosses four factors (per-group sample
#' size, AR(1) coefficient, innovation tail, imbalance ratio) and evaluates
#' three inferential layers (HAR-Wald, stationary block bootstrap,
#' hierarchical Bayesian HMC) on each replication.
#'
#' @section Data-generating process:
#' Two independent AR(1) series \eqn{x^{(1)}_t} of length \eqn{n_1=n} and
#' \eqn{x^{(2)}_t} of length \eqn{n_2 = \max(2, \mathrm{round}(n / \mathrm{imb}))}
#' are generated. The innovations are drawn from \code{rnorm},
#' scaled Student-\eqn{t_{(5)}}, scaled Student-\eqn{t_{(3)}} or scaled
#' Student-\eqn{t_{(2.1)}} so that the nominal innovation variance is one
#' when the law has a finite second moment. Group one has mean zero; group
#' two has mean \eqn{\Delta\cdot\sigma_\infty} where
#' \eqn{\sigma_\infty = 1/\sqrt{1 - \phi^2}}.
#'
#' @section Inferential layers:
#' \itemize{
#'   \item \strong{HAR-Wald (\code{"har"}).} Welch-style two-sample statistic
#'     \eqn{z = (\bar x_1 - \bar x_2) / \sqrt{\hat\sigma_1^2/n_1 + \hat\sigma_2^2/n_2}}
#'     with long-run variances estimated via \code{\link{htdv_lrv}} (QS
#'     kernel, Andrews bandwidth) on each group's demeaned series.
#'     Rejection at asymptotic \eqn{\chi^2_1} critical value.
#'   \item \strong{Stationary block bootstrap (\code{"boot"}).} Each group
#'     resampled independently using
#'     \code{\link{htdv_boot}} with \code{block_length = "auto"} (Patton-
#'     Politis-White). Percentile interval on
#'     \eqn{\bar x_1^* - \bar x_2^*}.
#'   \item \strong{Hierarchical Bayesian (\code{"bayes"}).} Two-sample AR(1)
#'     Stan model (\code{simstudy_two_sample.stan}); posterior interval on
#'     \eqn{\delta = \alpha_2 - \alpha_1}. A run is flagged
#'     \code{diag_pass = TRUE} only if \eqn{\hat R < 1.01}, bulk and tail
#'     ESS above \eqn{200}, and zero divergences.
#' }
#'
#' @param n_grid Integer vector of primary sample sizes \eqn{n_1}.
#' @param phi_grid Numeric vector of AR(1) coefficients in \eqn{(-1,1)}.
#' @param tail_grid Character vector; any subset of \code{"normal"},
#'   \code{"t5"}, \code{"t3"}, \code{"t2_1"}.
#' @param imb_grid Numeric vector of imbalance ratios \eqn{n_1/n_2}.
#' @param delta_grid Numeric vector of location shifts in units of
#'   \eqn{\sigma_\infty}.
#' @param R Integer number of Monte Carlo replications per cell.
#' @param seed Integer master seed.
#' @param n_cores Integer number of workers for cell-level parallelism.
#'   On non-Unix platforms parallelism falls back to serial.
#' @param layers Character vector; any subset of \code{c("har","boot","bayes")}.
#' @param bayes_chains Number of HMC chains per Bayesian fit.
#' @param bayes_iter Total HMC iterations (warmup + sampling).
#' @param bayes_warmup HMC warmup iterations.
#' @param boot_R Bootstrap replicates per call.
#' @param rope Length-2 numeric vector; ROPE for the Bayesian decision on
#'   the raw (non-standardized) delta.
#' @param alpha Significance level (nominal size of tests; \eqn{1-\alpha} is
#'   the coverage target for intervals).
#' @param out_dir Optional directory for per-cell incremental RDS results.
#'   Each cell is written to its own \code{cell_<id>.rds} so that concurrent
#'   workers cannot collide on a shared file. Final aggregation reads the
#'   directory back, so successful cells are preserved even if some workers
#'   later die. If \code{NULL}, no per-cell artifacts are written and the
#'   only result is the in-memory return value.
#' @param progress Logical; print a one-line status per completed cell.
#'
#' @return A data frame with one row per (cell, replication, layer). Columns:
#' \code{cell_id, n, phi, tail, imb, n1, n2, delta, sigma_inf, layer,
#' replicate, reject, ci_lo, ci_hi, estimate, covered, ci_length,
#' rhat_max, ess_min, divergences, diag_pass, runtime_sec}.
#'
#' @references
#' Kiefer, N.M., & Vogelsang, T.J. (2005). Econometric Theory 21(6):
#' 1130-1164.
#' Patton, A., Politis, D.N., & White, H. (2009). Econometric Reviews
#' 28(4): 372-375.
#' Kruschke, J.K. (2018). Advances in Methods and Practices in
#' Psychological Science 1(2): 270-280.
#'
#' @examples
#' \donttest{
#' res <- htdv_simstudy(n_grid = c(40, 80),
#'                      phi_grid = c(0, 0.6),
#'                      tail_grid = c("normal"),
#'                      imb_grid = c(1, 3),
#'                      delta_grid = c(0, 0.25),
#'                      R = 5L, n_cores = 1L,
#'                      layers = c("har", "boot"))
#' head(htdv_simstudy_summary(res))
#' }
#'
#' @export
htdv_simstudy <- function(n_grid = c(40L, 80L, 200L, 500L),
                          phi_grid = c(0, 0.3, 0.6, 0.85),
                          tail_grid = c("normal", "t5", "t3", "t2_1"),
                          imb_grid = c(1, 1.5, 3, 6),
                          delta_grid = c(0, 0.1, 0.25, 0.5),
                          R = 500L,
                          seed = 20260422L,
                          n_cores = 1L,
                          layers = c("har", "boot", "bayes"),
                          bayes_chains = 2L,
                          bayes_iter = 600L,
                          bayes_warmup = 300L,
                          boot_R = 999L,
                          rope = c(-0.1, 0.1),
                          alpha = 0.05,
                          out_dir = NULL,
                          progress = TRUE) {

  layers <- match.arg(layers, several.ok = TRUE,
                      choices = c("har", "boot", "bayes"))
  n_grid <- as.integer(n_grid)
  R <- as.integer(R)
  if (any(n_grid < 10L)) stop("n_grid entries must be >= 10.", call. = FALSE)
  if (R < 2L) stop("R must be >= 2.", call. = FALSE)
  if (!is.numeric(rope) || length(rope) != 2L || rope[1L] >= rope[2L])
    stop("'rope' must be a length-2 increasing numeric.", call. = FALSE)

  stan_model <- if ("bayes" %in% layers) {
    .load_stan_model("simstudy_two_sample")
  } else NULL

  grid <- expand.grid(n = n_grid, phi = phi_grid, tail = tail_grid,
                      imb = imb_grid, delta = delta_grid,
                      stringsAsFactors = FALSE)
  grid$cell_id <- seq_len(nrow(grid))

  if (!is.null(out_dir)) {
    dir.create(out_dir, recursive = TRUE, showWarnings = FALSE)
  }

  if (progress) {
    message(sprintf("htdv_simstudy: %d cells x %d replications x %d layers.",
                    nrow(grid), R, length(layers)))
  }

  cell_path <- function(i) {
    file.path(out_dir, sprintf("cell_%05d.rds", i))
  }

  run_one <- function(i) {
    if (!is.null(out_dir) && file.exists(cell_path(i))) {
      if (progress) {
        message(sprintf("  cell %d/%d | resumed from cache, skipped",
                        i, nrow(grid)))
      }
      return(invisible(NULL))
    }
    spec <- grid[i, , drop = FALSE]
    cell_seed <- .simstudy_cell_seed(seed, i)
    out <- .simstudy_run_cell(spec = spec, R = R, cell_seed = cell_seed,
                              layers = layers, stan_model = stan_model,
                              bayes_chains = bayes_chains,
                              bayes_iter = bayes_iter,
                              bayes_warmup = bayes_warmup,
                              boot_R = boot_R, rope = rope, alpha = alpha)
    if (!is.null(out_dir)) {
      .simstudy_append_cell(out, out_dir)
    }
    if (progress) {
      tot <- sum(out$runtime_sec, na.rm = TRUE)
      message(sprintf("  cell %d/%d | n=%d phi=%.2f tail=%s imb=%.2f delta=%.2f | %.1fs",
                      i, nrow(grid), spec$n, spec$phi, spec$tail,
                      spec$imb, spec$delta, tot))
    }
    if (is.null(out_dir)) out else invisible(NULL)
  }

  use_mc <- identical(.Platform$OS.type, "unix") && n_cores > 1L
  worker_results <- if (use_mc) {
    parallel::mclapply(seq_len(nrow(grid)), run_one,
                       mc.cores = n_cores, mc.preschedule = FALSE)
  } else {
    lapply(seq_len(nrow(grid)), run_one)
  }

  n_worker_errors <- sum(vapply(worker_results,
                                function(x) inherits(x, "try-error"),
                                logical(1L)))
  if (n_worker_errors > 0L) {
    warning(sprintf("htdv_simstudy: %d/%d workers errored. Surviving cells are preserved on disk.",
                    n_worker_errors, length(worker_results)),
            call. = FALSE)
  }

  if (!is.null(out_dir)) {
    files <- list.files(out_dir, pattern = "^cell_\\d+\\.rds$",
                        full.names = TRUE)
    if (!length(files)) {
      stop("No per-cell result files found in '", out_dir, "'.", call. = FALSE)
    }
    parts <- lapply(files, function(f) {
      tryCatch(readRDS(f), error = function(e) NULL)
    })
    ok <- vapply(parts, is.data.frame, logical(1L))
    if (sum(ok) < length(parts)) {
      warning(sprintf("htdv_simstudy: %d cell file(s) unreadable; skipped.",
                      sum(!ok)), call. = FALSE)
    }
    do.call(rbind, parts[ok])
  } else {
    df_ok <- vapply(worker_results, is.data.frame, logical(1L))
    if (!any(df_ok)) {
      stop("No worker returned usable data and out_dir was NULL.",
           call. = FALSE)
    }
    do.call(rbind, worker_results[df_ok])
  }
}

#' Flag Simulation-Study Cells in the Limit-of-Identification Zone
#'
#' Reads an aggregated \code{\link{htdv_simstudy_summary}} output and returns
#' the subset of cells where the Bayesian diagnostic-pass rate falls below a
#' user-set threshold. Such cells are typically located in the corner of the
#' design where strong autocorrelation meets a small sample (high \eqn{\phi}
#' and low \eqn{n}); the AR(1) likelihood there approaches its limit of
#' identification, MCMC diagnostics flag the resulting posterior, and
#' practitioners should treat the cell with extra caution—either by
#' increasing HMC iterations, tightening the priors on \eqn{\phi}, or
#' falling back to the conformal anchor whose validity does not require
#' identifiability.
#'
#' @param summ Data frame from \code{\link{htdv_simstudy_summary}}.
#' @param threshold Minimum acceptable Bayesian \code{diag_pass_rate}.
#'   Default \code{0.7}.
#'
#' @return A data frame with the columns \code{n, phi, tail, imb, delta,
#'   layer, diag_pass_rate}, restricted to Bayesian rows whose pass rate is
#'   below \code{threshold}, plus an integer column \code{flag_index}
#'   (sequential identifier of flagged cells, useful for cross-referencing
#'   in reports and in vignettes).
#'
#' @examples
#' \donttest{
#' res <- htdv_simstudy(n_grid = c(40, 200),
#'                      phi_grid = c(0, 0.85),
#'                      tail_grid = "normal",
#'                      imb_grid = c(1, 6),
#'                      delta_grid = c(0, 0.25),
#'                      R = 50L)
#' summ <- htdv_simstudy_summary(res)
#' htdv_simstudy_warnings(summ, threshold = 0.7)
#' }
#'
#' @export
htdv_simstudy_warnings <- function(summ, threshold = 0.7) {
  if (!is.data.frame(summ)) stop("'summ' must be a data frame.", call. = FALSE)
  required <- c("n", "phi", "tail", "imb", "delta", "layer", "diag_pass_rate")
  missing_cols <- setdiff(required, names(summ))
  if (length(missing_cols)) {
    stop("'summ' is missing columns: ", paste(missing_cols, collapse = ", "),
         call. = FALSE)
  }
  if (!is.numeric(threshold) || length(threshold) != 1L ||
      threshold <= 0 || threshold >= 1) {
    stop("'threshold' must be a scalar in (0, 1).", call. = FALSE)
  }
  bayes <- summ[summ$layer == "bayes", , drop = FALSE]
  flagged <- bayes[is.finite(bayes$diag_pass_rate) &
                   bayes$diag_pass_rate < threshold, required, drop = FALSE]
  flagged <- flagged[order(flagged$diag_pass_rate), , drop = FALSE]
  rownames(flagged) <- NULL
  flagged$flag_index <- seq_len(nrow(flagged))
  flagged
}

#' Aggregate a Simulation-Study Output
#'
#' Computes empirical size (or rejection rate), coverage, and mean interval
#' length per (cell x layer) from the raw output of \code{\link{htdv_simstudy}}.
#'
#' @param res Data frame returned by \code{htdv_simstudy}.
#' @param alpha Nominal size used in the study (must match what
#'   \code{htdv_simstudy} was called with).
#'
#' @return A data frame with one row per (cell, layer) and columns
#' \code{n, phi, tail, imb, delta, layer, n_reps, reject_rate, coverage,
#' mean_ci_length, mean_runtime, diag_pass_rate}.
#'
#' @examples
#' \donttest{
#' res <- htdv_simstudy(n_grid = c(40), phi_grid = c(0.3),
#'                      tail_grid = "normal", imb_grid = 1,
#'                      delta_grid = c(0, 0.3), R = 5,
#'                      layers = c("har"))
#' htdv_simstudy_summary(res)
#' }
#'
#' @export
htdv_simstudy_summary <- function(res, alpha = 0.05) {
  if (!is.data.frame(res)) stop("'res' must be a data frame.", call. = FALSE)
  by_cols <- c("n", "phi", "tail", "imb", "delta", "layer")
  key <- do.call(paste, c(res[by_cols], sep = "|"))
  split_res <- split(res, key)
  out <- lapply(split_res, function(d) {
    data.frame(
      n = d$n[1L], phi = d$phi[1L], tail = d$tail[1L],
      imb = d$imb[1L], delta = d$delta[1L], layer = d$layer[1L],
      n_reps = nrow(d),
      reject_rate = mean(d$reject, na.rm = TRUE),
      coverage = mean(d$covered, na.rm = TRUE),
      mean_ci_length = mean(d$ci_length, na.rm = TRUE),
      mean_runtime = mean(d$runtime_sec, na.rm = TRUE),
      diag_pass_rate = if (any(!is.na(d$diag_pass)))
        mean(d$diag_pass, na.rm = TRUE) else NA_real_,
      stringsAsFactors = FALSE
    )
  })
  agg <- do.call(rbind, out)
  rownames(agg) <- NULL
  agg
}

# --- Internal helpers --------------------------------------------------------

.simstudy_cell_seed <- function(master, i) {
  as.integer((as.numeric(master) + 1e4 * i) %% .Machine$integer.max)
}

.simstudy_rep_seed <- function(cell_seed, r) {
  as.integer((as.numeric(cell_seed) + r) %% .Machine$integer.max)
}

.simstudy_append_cell <- function(rows, dir) {
  cell_id <- rows$cell_id[1L]
  tmp <- tempfile(tmpdir = dir, pattern = sprintf("cell_%05d_", cell_id),
                  fileext = ".rds.tmp")
  saveRDS(rows, file = tmp)
  final <- file.path(dir, sprintf("cell_%05d.rds", cell_id))
  file.rename(tmp, final)
  invisible(TRUE)
}

.simstudy_gen_ar1 <- function(n, phi, tail, mean_shift = 0, burn = 200L) {
  m <- n + burn
  eps <- switch(tail,
    "normal" = stats::rnorm(m),
    "t5"     = stats::rt(m, df = 5) / sqrt(5 / 3),
    "t3"     = stats::rt(m, df = 3) / sqrt(3 / 1),
    "t2_1"   = stats::rt(m, df = 2.1) / sqrt(2.1 / 0.1),
    stop("unknown tail: ", tail, call. = FALSE)
  )
  x <- numeric(m)
  x[1L] <- eps[1L]
  for (t in 2:m) x[t] <- phi * x[t - 1L] + eps[t]
  x[(burn + 1L):m] + mean_shift
}

.simstudy_run_cell <- function(spec, R, cell_seed, layers, stan_model,
                               bayes_chains, bayes_iter, bayes_warmup,
                               boot_R, rope, alpha) {
  n1 <- spec$n
  n2 <- max(2L, as.integer(round(spec$n / spec$imb)))
  sigma_inf <- 1 / sqrt(max(1 - spec$phi^2, 1e-12))
  true_delta <- spec$delta * sigma_inf
  rows <- vector("list", R * length(layers))
  k <- 0L
  for (r in seq_len(R)) {
    rep_seed <- .simstudy_rep_seed(cell_seed, r)
    set.seed(rep_seed)
    x1 <- .simstudy_gen_ar1(n1, spec$phi, spec$tail, 0)
    x2 <- .simstudy_gen_ar1(n2, spec$phi, spec$tail, true_delta)

    for (lyr in layers) {
      t0 <- proc.time()[["elapsed"]]
      layer_out <- tryCatch(
        switch(lyr,
          "har"   = .simstudy_layer_har(x1, x2, alpha),
          "boot"  = .simstudy_layer_boot(x1, x2, boot_R, alpha, rep_seed),
          "bayes" = .simstudy_layer_bayes(x1, x2, stan_model,
                                          bayes_chains, bayes_iter,
                                          bayes_warmup, rope, alpha,
                                          rep_seed)
        ),
        error = function(e) list(reject = NA, ci_lo = NA_real_,
                                 ci_hi = NA_real_, estimate = NA_real_,
                                 rhat_max = NA_real_, ess_min = NA_real_,
                                 divergences = NA_integer_,
                                 diag_pass = NA, err = conditionMessage(e))
      )
      elapsed <- proc.time()[["elapsed"]] - t0
      covered <- if (is.finite(layer_out$ci_lo) && is.finite(layer_out$ci_hi))
        (true_delta >= layer_out$ci_lo) && (true_delta <= layer_out$ci_hi)
      else NA
      ci_length <- if (is.finite(layer_out$ci_lo) && is.finite(layer_out$ci_hi))
        layer_out$ci_hi - layer_out$ci_lo else NA_real_

      k <- k + 1L
      rows[[k]] <- data.frame(
        cell_id   = spec$cell_id,
        n         = n1,
        phi       = spec$phi,
        tail      = spec$tail,
        imb       = spec$imb,
        n1        = n1, n2 = n2,
        delta     = spec$delta,
        sigma_inf = sigma_inf,
        layer     = lyr,
        replicate = r,
        reject    = as.logical(layer_out$reject),
        ci_lo     = layer_out$ci_lo,
        ci_hi     = layer_out$ci_hi,
        estimate  = layer_out$estimate,
        covered   = covered,
        ci_length = ci_length,
        rhat_max    = if (!is.null(layer_out$rhat_max)) layer_out$rhat_max
                      else NA_real_,
        ess_min     = if (!is.null(layer_out$ess_min)) layer_out$ess_min
                      else NA_real_,
        divergences = if (!is.null(layer_out$divergences))
                      layer_out$divergences else NA_integer_,
        diag_pass   = if (!is.null(layer_out$diag_pass)) layer_out$diag_pass
                      else NA,
        runtime_sec = elapsed,
        stringsAsFactors = FALSE
      )
    }
  }
  do.call(rbind, rows)
}

.simstudy_layer_har <- function(x1, x2, alpha) {
  # Sign convention: estimand is delta = E[X2] - E[X1], matching the Stan
  # layer's parameter delta = alpha2 - alpha1. Coverage of CI is checked
  # against the same true_delta = +shift used by the data generator.
  n1 <- length(x1); n2 <- length(x2)
  m1 <- mean(x1); m2 <- mean(x2)
  lrv1 <- htdv_lrv(x1, kernel = "qs", bandwidth = "andrews")$lrv
  lrv2 <- htdv_lrv(x2, kernel = "qs", bandwidth = "andrews")$lrv
  se <- sqrt(lrv1 / n1 + lrv2 / n2)
  est <- m2 - m1
  z <- est / se
  z_crit <- stats::qnorm(1 - alpha / 2)
  list(reject = abs(z) > z_crit,
       ci_lo = est - z_crit * se,
       ci_hi = est + z_crit * se,
       estimate = est,
       rhat_max = NA_real_, ess_min = NA_real_,
       divergences = NA_integer_, diag_pass = NA)
}

.simstudy_layer_boot <- function(x1, x2, R, alpha, seed) {
  # Sign convention: see .simstudy_layer_har. delta = E[X2] - E[X1].
  n1 <- length(x1); n2 <- length(x2)
  b1 <- .ppw_block_length(x1)
  b2 <- .ppw_block_length(x2)
  set.seed(seed)
  diffs <- numeric(R)
  for (r in seq_len(R)) {
    xb1 <- .stationary_bootstrap_sample(x1, p = 1 / b1)
    xb2 <- .stationary_bootstrap_sample(x2, p = 1 / b2)
    diffs[r] <- mean(xb2) - mean(xb1)
  }
  est <- mean(x2) - mean(x1)
  q <- as.numeric(stats::quantile(diffs, probs = c(alpha / 2, 1 - alpha / 2),
                                  names = FALSE))
  list(reject = (q[1L] > 0) || (q[2L] < 0),
       ci_lo = q[1L], ci_hi = q[2L], estimate = est,
       rhat_max = NA_real_, ess_min = NA_real_,
       divergences = NA_integer_, diag_pass = NA)
}

.simstudy_layer_bayes <- function(x1, x2, stan_model, chains, iter, warmup,
                                  rope, alpha, seed) {
  sd <- list(N1 = length(x1), N2 = length(x2), x1 = x1, x2 = x2)
  fit <- suppressWarnings(
    rstan::sampling(stan_model, data = sd, chains = chains,
                    iter = iter, warmup = warmup, refresh = 0L,
                    seed = seed,
                    control = list(adapt_delta = 0.95,
                                   max_treedepth = 12L))
  )
  draws <- as.numeric(rstan::extract(fit, pars = "delta",
                                     permuted = TRUE)$delta)
  ci <- as.numeric(stats::quantile(draws,
                                   probs = c(alpha / 2, 1 - alpha / 2),
                                   names = FALSE))
  est <- mean(draws)
  rej_ci <- (ci[1L] > 0) || (ci[2L] < 0)
  sf_sum <- rstan::summary(fit,
                           pars = c("alpha1","alpha2","phi","sigma","delta"),
                           probs = c(0.025, 0.975))$summary
  rhat_max <- max(sf_sum[, "Rhat"], na.rm = TRUE)
  ess_min <- min(sf_sum[, "n_eff"], na.rm = TRUE)
  sp <- rstan::get_sampler_params(fit, inc_warmup = FALSE)
  divs <- sum(sapply(sp, function(p) sum(p[, "divergent__"])))
  diag_pass <- is.finite(rhat_max) && rhat_max < 1.01 &&
               is.finite(ess_min) && ess_min > 200 && divs == 0L
  list(reject = rej_ci,
       ci_lo = ci[1L], ci_hi = ci[2L], estimate = est,
       rhat_max = rhat_max, ess_min = ess_min,
       divergences = as.integer(divs), diag_pass = diag_pass)
}

#' Long-Run Variance Estimation (HAC)
#'
#' Heteroskedasticity-and-autocorrelation-consistent long-run variance with
#' data-driven Andrews (1991) optimal bandwidth, Bartlett, Parzen, and
#' Quadratic-Spectral kernels.
#'
#' @param x Numeric vector of observations.
#' @param kernel Kernel choice: \code{"bartlett"}, \code{"parzen"}, or
#'   \code{"qs"} (Quadratic-Spectral).
#' @param bandwidth Either \code{"andrews"} (data-driven) or a positive
#'   numeric scalar.
#' @param prewhiten Logical; if \code{TRUE}, AR(1) prewhiten-recolor
#'   (Andrews-Monahan 1992).
#'
#' @return A list with components \code{lrv} (long-run variance estimate),
#'   \code{bandwidth}, \code{kernel}, and \code{ar_coef} (only if prewhitened).
#'
#' @references
#' Andrews, D.W.K. (1991). Heteroskedasticity and Autocorrelation Consistent
#' Covariance Matrix Estimation. Econometrica 59(3): 817-858.
#' Newey, W.K. & West, K.D. (1987). A Simple, Positive Semi-Definite
#' Heteroskedasticity and Autocorrelation Consistent Covariance Matrix.
#' Econometrica 55(3): 703-708.
#'
#' @examples
#' set_seed_user <- 42
#' x <- arima.sim(model = list(ar = 0.6), n = 200, rand.gen = rnorm)
#' out <- htdv_lrv(as.numeric(x), kernel = "qs", bandwidth = "andrews")
#' out$lrv
#'
#' @export
htdv_lrv <- function(x, kernel = c("bartlett", "parzen", "qs"),
                     bandwidth = "andrews", prewhiten = FALSE) {
  kernel <- match.arg(kernel)
  .assert_numeric_vector(x, "x")
  n <- length(x)
  u <- .demean(x)
  ar_coef <- NA_real_
  if (isTRUE(prewhiten)) {
    fit <- stats::ar(u, aic = FALSE, order.max = 1L, method = "ols")
    if (length(fit$ar) >= 1L) {
      ar_coef <- as.numeric(fit$ar[1L])
      u <- as.numeric(stats::na.omit(fit$resid))
      n <- length(u)
    }
  }
  h <- if (identical(bandwidth, "andrews")) {
    .andrews_bandwidth(u, kernel = kernel)
  } else {
    .assert_positive_scalar(bandwidth, "bandwidth")
    as.numeric(bandwidth)
  }
  acv <- .auto_cov(u, lag_max = n - 1L)
  lrv <- .kernel_sum(acv, h = h, kernel = kernel, n = n)
  if (isTRUE(prewhiten) && is.finite(ar_coef) && abs(ar_coef) < 0.999) {
    lrv <- lrv / (1 - ar_coef)^2
  }
  list(lrv = lrv, bandwidth = h, kernel = kernel, ar_coef = ar_coef)
}

.auto_cov <- function(u, lag_max) {
  n <- length(u)
  lag_max <- min(lag_max, n - 1L)
  out <- numeric(lag_max + 1L)
  for (k in 0:lag_max) {
    out[k + 1L] <- sum(u[1:(n - k)] * u[(1 + k):n]) / n
  }
  out
}

.kernel_weight <- function(z, kernel) {
  if (kernel == "bartlett") {
    pmax(0, 1 - abs(z))
  } else if (kernel == "parzen") {
    az <- abs(z)
    w <- ifelse(az <= 0.5, 1 - 6 * az^2 + 6 * az^3,
                ifelse(az <= 1, 2 * (1 - az)^3, 0))
    w
  } else if (kernel == "qs") {
    out <- numeric(length(z))
    nz <- abs(z) > .Machine$double.eps
    zz <- z[nz]
    arg <- 6 * pi * zz / 5
    out[nz] <- (25 / (12 * pi^2 * zz^2)) *
      (sin(arg) / arg - cos(arg))
    out[!nz] <- 1
    out
  } else {
    stop("Unknown kernel.", call. = FALSE)
  }
}

.kernel_sum <- function(acv, h, kernel, n) {
  lag_max <- length(acv) - 1L
  k_vec <- seq_len(lag_max)
  z <- k_vec / h
  w <- .kernel_weight(z, kernel)
  val <- acv[1L] + 2 * sum(w * acv[-1L])
  max(val, .Machine$double.eps)
}

.andrews_bandwidth <- function(u, kernel) {
  n <- length(u)
  fit <- stats::ar(u, aic = FALSE, order.max = 1L, method = "ols")
  rho <- if (length(fit$ar) >= 1L) as.numeric(fit$ar[1L]) else 0
  sig2 <- if (length(fit$var.pred) >= 1L) as.numeric(fit$var.pred) else .safe_var(u)
  if (!is.finite(rho) || abs(rho) >= 0.999) rho <- 0.99 * sign(rho)
  num <- 4 * rho^2 * sig2^2
  den <- (1 - rho)^6 * (1 + rho)^2
  alpha2 <- num / max(den, .Machine$double.eps)
  num4 <- 4 * rho^2 * sig2^2
  den4 <- (1 - rho)^8
  alpha4 <- num4 / max(den4, .Machine$double.eps)
  h <- switch(kernel,
    bartlett = 1.1447 * (alpha2 * n)^(1 / 3),
    parzen   = 2.6614 * (alpha4 * n)^(1 / 5),
    qs       = 1.3221 * (alpha4 * n)^(1 / 5)
  )
  max(h, 1)
}

test_that("htdv_lrv returns a positive LRV for iid data", {
  set.seed(17)
  x <- stats::rnorm(200)
  out <- htdv_lrv(x, kernel = "qs", bandwidth = "andrews")
  expect_true(is.numeric(out$lrv))
  expect_gt(out$lrv, 0)
  expect_true(is.finite(out$lrv))
})

test_that("htdv_lrv inflates LRV for positively correlated data", {
  set.seed(18)
  x_ar <- stats::arima.sim(model = list(ar = 0.8), n = 500,
                           rand.gen = stats::rnorm)
  out_ar <- htdv_lrv(as.numeric(x_ar), kernel = "bartlett",
                     bandwidth = "andrews")
  set.seed(19)
  out_iid <- htdv_lrv(stats::rnorm(500), kernel = "bartlett",
                      bandwidth = "andrews")
  expect_gt(out_ar$lrv, out_iid$lrv)
})

test_that("htdv_lrv rejects invalid inputs", {
  expect_error(htdv_lrv(c(1, 2, NA)))
  expect_error(htdv_lrv(1))
  expect_error(htdv_lrv(stats::rnorm(50), bandwidth = -1))
})

test_that("htdv_lrv supports all three kernels", {
  set.seed(20)
  x <- stats::rnorm(100)
  for (k in c("bartlett", "parzen", "qs")) {
    out <- htdv_lrv(x, kernel = k, bandwidth = "andrews")
    expect_true(out$lrv > 0)
  }
})

.htdv_env <- new.env(parent = emptyenv())

.onLoad <- function(libname, pkgname) {
  assign("stan_cache", list(), envir = .htdv_env)
  invisible(NULL)
}

.onUnload <- function(libpath) {
  if (exists("stan_cache", envir = .htdv_env, inherits = FALSE)) {
    rm("stan_cache", envir = .htdv_env)
  }
  invisible(NULL)
}

#' MCMC Diagnostics for htdv_fit Objects
#'
#' Computes split-Rhat, bulk and tail ESS, E-BFMI, divergence count, and
#' treedepth saturation; all standard HMC diagnostics (Vehtari et al. 2021,
#' Betancourt 2016).
#'
#' @param fit An \code{htdv_fit} object.
#' @param rhat_threshold Convergence threshold for Rhat.
#' @param ess_threshold Minimum effective sample size.
#' @param bfmi_threshold Minimum acceptable energy-BFMI.
#'
#' @return A list with \code{rhat}, \code{ess_bulk}, \code{ess_tail},
#'   \code{bfmi}, \code{divergences}, \code{max_treedepth},
#'   \code{passed} (logical), and \code{failures} (character vector).
#'
#' @references
#' Vehtari, A., Gelman, A., Simpson, D., Carpenter, B., & Buerkner, P.-C.
#' (2021). Rank-normalization, folding, and localization: An improved Rhat
#' for assessing convergence of MCMC. Bayesian Analysis 16(2): 667-718.
#' Betancourt, M. (2016). Diagnosing Suboptimal Cotangent Disintegrations.
#' <doi:10.48550/arXiv.1604.00695>.
#'
#' @examples
#' \donttest{
#' x <- rnorm(50)
#' fit <- htdv_fit(x, model = "tac", chains = 2, iter = 500,
#'                 refresh = 0, seed = 1)
#' htdv_diagnostics(fit)$passed
#' }
#'
#' @export
htdv_diagnostics <- function(fit, rhat_threshold = 1.01,
                             ess_threshold = 400, bfmi_threshold = 0.3) {
  if (!inherits(fit, "htdv_fit")) {
    stop("'fit' must be an 'htdv_fit' object.", call. = FALSE)
  }
  sf <- fit$stanfit
  mon <- rstan::monitor(sf, print = FALSE, warmup = 0)
  rh <- mon[, "Rhat"]
  essb <- mon[, "Bulk_ESS"]
  esst <- mon[, "Tail_ESS"]
  sp <- rstan::get_sampler_params(sf, inc_warmup = FALSE)
  div <- sum(sapply(sp, function(p) sum(p[, "divergent__"])))
  tdmax <- max(sapply(sp, function(p) max(p[, "treedepth__"])))
  energy_bfmi <- vapply(sp, function(p) {
    e <- p[, "energy__"]
    var_e <- stats::var(e)
    if (!is.finite(var_e) || var_e <= 0) return(NA_real_)
    sum(diff(e)^2) / (length(e) - 1L) / var_e
  }, numeric(1L))
  fails <- character(0L)
  if (any(rh > rhat_threshold, na.rm = TRUE)) {
    fails <- c(fails, sprintf("Rhat > %.3f on some parameter", rhat_threshold))
  }
  if (any(essb < ess_threshold, na.rm = TRUE)) {
    fails <- c(fails, "Bulk ESS below threshold")
  }
  if (any(esst < ess_threshold, na.rm = TRUE)) {
    fails <- c(fails, "Tail ESS below threshold")
  }
  if (div > 0L) {
    fails <- c(fails, sprintf("%d post-warmup divergences", div))
  }
  if (any(is.na(energy_bfmi)) || any(energy_bfmi < bfmi_threshold, na.rm = TRUE)) {
    fails <- c(fails, "E-BFMI below threshold")
  }
  list(rhat = rh, ess_bulk = essb, ess_tail = esst,
       bfmi = energy_bfmi, divergences = div,
       max_treedepth = tdmax,
       passed = length(fails) == 0L,
       failures = fails)
}

#' Posterior-Predictive Checks on Dependence Statistics
#'
#' Replicates autocorrelation, Ljung-Box, spectral discrepancy, and a
#' detrended-fluctuation Hurst exponent under the posterior and computes
#' posterior-predictive \code{p}-values.
#'
#' @param fit An \code{htdv_fit} object whose posterior-predictive replicates
#'   are stored as parameter \code{x_rep}.
#' @param x_obs The observed data vector.
#' @param lag_max Maximum lag for autocorrelation.
#'
#' @return A list with \code{p_acf}, \code{p_ljung_box}, \code{p_spectral},
#'   \code{p_hurst}.
#'
#' @examples
#' \donttest{
#' x <- rnorm(50)
#' fit <- htdv_fit(x, model = "tac", chains = 2, iter = 500,
#'                 refresh = 0, seed = 1)
#' htdv_ppc(fit, x)
#' }
#'
#' @export
htdv_ppc <- function(fit, x_obs, lag_max = 10L) {
  if (!inherits(fit, "htdv_fit")) {
    stop("'fit' must be an 'htdv_fit' object.", call. = FALSE)
  }
  .assert_numeric_vector(x_obs, "x_obs")
  sf <- fit$stanfit
  pars <- sf@model_pars
  if (!"x_rep" %in% pars) {
    stop("Posterior-predictive replicate 'x_rep' not found.", call. = FALSE)
  }
  xrep <- rstan::extract(sf, pars = "x_rep", permuted = TRUE)$x_rep
  t_obs_acf <- sum(stats::acf(x_obs, lag.max = lag_max, plot = FALSE)$acf[-1L]^2)
  t_obs_lb <- .ljung_box(x_obs, lag = max(1L, floor(log(length(x_obs)))))
  t_obs_sp <- .spectral_discrepancy(x_obs)
  t_obs_h <- .dfa_hurst(x_obs)
  t_rep_acf <- apply(xrep, 1L, function(xr) {
    sum(stats::acf(xr, lag.max = lag_max, plot = FALSE)$acf[-1L]^2)
  })
  t_rep_lb <- apply(xrep, 1L, function(xr) {
    .ljung_box(xr, lag = max(1L, floor(log(length(xr)))))
  })
  t_rep_sp <- apply(xrep, 1L, .spectral_discrepancy)
  t_rep_h <- apply(xrep, 1L, .dfa_hurst)
  list(p_acf = mean(t_rep_acf >= t_obs_acf),
       p_ljung_box = mean(t_rep_lb >= t_obs_lb),
       p_spectral = mean(t_rep_sp >= t_obs_sp),
       p_hurst = mean(t_rep_h >= t_obs_h))
}

.ljung_box <- function(x, lag = 10L) {
  n <- length(x)
  r <- stats::acf(x, lag.max = lag, plot = FALSE)$acf[-1L]
  n * (n + 2) * sum(r^2 / (n - seq_along(r)))
}

.spectral_discrepancy <- function(x) {
  sp <- stats::spec.pgram(x, plot = FALSE, taper = 0, detrend = FALSE,
                          demean = TRUE)
  mean(abs(sp$spec - mean(sp$spec))^2)
}

.dfa_hurst <- function(x) {
  n <- length(x)
  if (n < 16L) return(NA_real_)
  y <- cumsum(x - mean(x))
  scales <- unique(round(exp(seq(log(4), log(n / 4), length.out = 10))))
  scales <- scales[scales >= 4L & scales <= n / 4L]
  if (length(scales) < 2L) return(NA_real_)
  fs <- numeric(length(scales))
  for (i in seq_along(scales)) {
    s <- scales[i]
    k <- floor(n / s)
    if (k < 1L) { fs[i] <- NA_real_; next }
    rs <- numeric(k)
    for (j in seq_len(k)) {
      seg <- y[((j - 1L) * s + 1L):(j * s)]
      tseg <- seq_len(s)
      beta <- stats::.lm.fit(cbind(1, tseg), seg)$coefficients
      fit <- beta[1L] + beta[2L] * tseg
      rs[j] <- mean((seg - fit)^2)
    }
    fs[i] <- sqrt(mean(rs, na.rm = TRUE))
  }
  ok <- is.finite(fs) & fs > 0
  if (sum(ok) < 2L) return(NA_real_)
  coef <- stats::.lm.fit(cbind(1, log(scales[ok])), log(fs[ok]))$coefficients
  as.numeric(coef[2L])
}

.assert_numeric_vector <- function(x, name = "x", min_len = 2L) {
  if (!is.numeric(x) || !is.null(dim(x))) {
    stop(sprintf("'%s' must be a numeric vector.", name), call. = FALSE)
  }
  if (length(x) < min_len) {
    stop(sprintf("'%s' must have length >= %d.", name, min_len), call. = FALSE)
  }
  if (any(!is.finite(x))) {
    stop(sprintf("'%s' must not contain NA/NaN/Inf.", name), call. = FALSE)
  }
  invisible(TRUE)
}

.assert_positive_scalar <- function(x, name = "x") {
  if (!is.numeric(x) || length(x) != 1L || !is.finite(x) || x <= 0) {
    stop(sprintf("'%s' must be a positive finite scalar.", name), call. = FALSE)
  }
  invisible(TRUE)
}

.assert_probability <- function(p, name = "p") {
  if (!is.numeric(p) || length(p) != 1L || !is.finite(p) || p <= 0 || p >= 1) {
    stop(sprintf("'%s' must be a scalar in (0, 1).", name), call. = FALSE)
  }
  invisible(TRUE)
}

.has_package <- function(pkg) {
  requireNamespace(pkg, quietly = TRUE)
}

.require_suggested <- function(pkg) {
  if (!.has_package(pkg)) {
    stop(sprintf(
      "Package '%s' is required for this function. Please install it.",
      pkg
    ), call. = FALSE)
  }
  invisible(TRUE)
}

.with_restored_options <- function(expr) {
  old <- options()
  on.exit(options(old), add = TRUE)
  force(expr)
}

.load_stan_model <- function(name) {
  cache <- get("stan_cache", envir = .htdv_env)
  if (!is.null(cache[[name]])) return(cache[[name]])
  stan_file <- system.file("stan", paste0(name, ".stan"), package = "HTDV")
  if (!nzchar(stan_file)) {
    stop(sprintf("Stan file '%s.stan' not found in HTDV/inst/stan.", name),
         call. = FALSE)
  }
  model <- rstan::stan_model(file = stan_file, auto_write = FALSE)
  cache[[name]] <- model
  assign("stan_cache", cache, envir = .htdv_env)
  model
}

.demean <- function(x) x - mean(x)

.safe_var <- function(x) {
  v <- stats::var(x)
  if (!is.finite(v) || v <= 0) 1e-12 else v
}

#' Region-of-Practical-Equivalence (ROPE) Decision
#'
#' Posterior decision under Kruschke's (2018) Region of Practical Equivalence.
#'
#' @param draws Numeric vector of posterior draws of the parameter of
#'   interest.
#' @param rope Length-2 numeric vector \code{c(lower, upper)} defining the
#'   practical-equivalence region.
#' @param level Credibility level for the highest-density interval.
#'
#' @return A list with \code{decision} (\code{"accept"}, \code{"reject"},
#'   \code{"undecided"}), \code{hdi}, \code{rope}, and \code{prob_in_rope}.
#'
#' @references
#' Kruschke, J.K. (2018). Rejecting or Accepting Parameter Values in Bayesian
#' Estimation. Advances in Methods and Practices in Psychological Science
#' 1(2): 270-280.
#'
#' @examples
#' draws <- rnorm(4000, mean = 0.05, sd = 0.1)
#' htdv_rope(draws, rope = c(-0.1, 0.1), level = 0.95)$decision
#'
#' @export
htdv_rope <- function(draws, rope, level = 0.95) {
  .assert_numeric_vector(draws, "draws", min_len = 100L)
  if (!is.numeric(rope) || length(rope) != 2L || rope[1L] >= rope[2L]) {
    stop("'rope' must be a length-2 numeric vector with rope[1] < rope[2].",
         call. = FALSE)
  }
  .assert_probability(level, "level")
  hdi <- .hdi(draws, level = level)
  inside <- (hdi[1L] >= rope[1L]) && (hdi[2L] <= rope[2L])
  outside <- (hdi[2L] < rope[1L]) || (hdi[1L] > rope[2L])
  decision <- if (inside) "accept" else if (outside) "reject" else "undecided"
  prob_in <- mean(draws >= rope[1L] & draws <= rope[2L])
  list(decision = decision, hdi = hdi, rope = rope,
       level = level, prob_in_rope = prob_in)
}

.hdi <- function(draws, level = 0.95) {
  s <- sort(draws)
  n <- length(s)
  k <- max(1L, floor(level * n))
  if (k >= n) return(c(s[1L], s[n]))
  widths <- s[(k + 1L):n] - s[1L:(n - k)]
  i <- which.min(widths)
  c(s[i], s[i + k])
}

#' Bridge-Sampling Bayes Factor
#'
#' Computes the Bayes factor between two fitted \code{\link{htdv_fit}}
#' objects via bridge sampling. Requires the \code{bridgesampling} package.
#'
#' @param fit1 Model in the numerator.
#' @param fit0 Model in the denominator.
#' @param ... Additional arguments passed to
#'   \code{bridgesampling::bridge_sampler}.
#'
#' @return A list with \code{bf10}, \code{log_bf10}, \code{logml1},
#'   \code{logml0}, and \code{error_percentage}.
#'
#' @references
#' Meng, X.-L., & Wong, W.H. (1996). Simulating Ratios of Normalizing
#' Constants. Statistica Sinica 6(4): 831-860.
#' Gronau, Q.F. et al. (2017). A Tutorial on Bridge Sampling. JMP 81: 80-97.
#'
#' @examples
#' \donttest{
#' if (requireNamespace("bridgesampling", quietly = TRUE)) {
#'   x <- rnorm(50)
#'   f1 <- htdv_fit(x, model = "tac", chains = 2, iter = 500,
#'                  refresh = 0, seed = 1)
#'   f0 <- htdv_fit(x, model = "tac", chains = 2, iter = 500,
#'                  refresh = 0, seed = 2)
#'   htdv_bf(f1, f0)$bf10
#' }
#' }
#'
#' @export
htdv_bf <- function(fit1, fit0, ...) {
  .require_suggested("bridgesampling")
  if (!inherits(fit1, "htdv_fit") || !inherits(fit0, "htdv_fit")) {
    stop("'fit1' and 'fit0' must be 'htdv_fit' objects.", call. = FALSE)
  }
  bs1 <- bridgesampling::bridge_sampler(fit1$stanfit, silent = TRUE, ...)
  bs0 <- bridgesampling::bridge_sampler(fit0$stanfit, silent = TRUE, ...)
  log_bf <- bs1$logml - bs0$logml
  list(bf10 = exp(log_bf), log_bf10 = log_bf,
       logml1 = bs1$logml, logml0 = bs0$logml,
       error_percentage = c(bridgesampling::error_measures(bs1)$percentage,
                            bridgesampling::error_measures(bs0)$percentage))
}

#' WAIC and Leave-Future-Out Cross-Validation
#'
#' Computes the Widely Applicable Information Criterion (Watanabe 2010) and
#' leave-future-out expected log predictive density (Buerkner-Gabry-Vehtari
#' 2020) from a fitted \code{\link{htdv_fit}} object.
#'
#' @param fit An \code{htdv_fit} object with pointwise log-likelihood stored
#'   as parameter \code{log_lik}.
#' @param L Block-prefix length; leave-future-out starts at time \code{L+1}.
#'   Default \code{floor(n/5)}.
#' @param k_threshold Pareto-k refresh threshold; when exceeded, importance
#'   sampling is refreshed.
#'
#' @return A list with \code{waic}, \code{elpd_lfo}, \code{elpd_se},
#'   \code{pareto_k}, and \code{refresh_times}.
#'
#' @references
#' Watanabe, S. (2010). Asymptotic Equivalence of Bayes Cross Validation and
#' Widely Applicable Information Criterion. JMLR 11: 3571-3594.
#' Buerkner, P.-C., Gabry, J., & Vehtari, A. (2020). Approximate
#' Leave-Future-Out Cross-Validation for Bayesian Time Series Models. Journal
#' of Statistical Computation and Simulation 90(14): 2499-2523.
#'
#' @examples
#' \donttest{
#' if (requireNamespace("loo", quietly = TRUE)) {
#'   x <- rnorm(50)
#'   fit <- htdv_fit(x, model = "tac", chains = 2, iter = 500,
#'                   refresh = 0, seed = 1)
#'   htdv_waic_lfo(fit)$waic
#' }
#' }
#'
#' @export
htdv_waic_lfo <- function(fit, L = NULL, k_threshold = 0.7) {
  .require_suggested("loo")
  if (!inherits(fit, "htdv_fit")) {
    stop("'fit' must be an 'htdv_fit' object.", call. = FALSE)
  }
  ll <- .extract_log_lik(fit)
  if (is.null(ll)) {
    stop("Log-likelihood 'log_lik' not found in the Stan fit.", call. = FALSE)
  }
  n <- ncol(ll)
  if (is.null(L)) L <- max(1L, floor(n / 5L))
  w <- loo::waic(ll)
  elpd_lfo <- NA_real_
  elpd_se <- NA_real_
  pareto_k <- rep(NA_real_, n)
  refreshes <- integer(0L)
  if (L < n) {
    elpds <- numeric(n - L)
    for (t in (L + 1L):n) {
      lw <- .log_weights_lfo(ll, t, L)
      pk <- .pareto_k_fast(lw)
      pareto_k[t] <- pk
      if (!is.finite(pk) || pk > k_threshold) {
        refreshes <- c(refreshes, t)
        lw <- rep(0, length(lw))
      }
      elpds[t - L] <- .log_mean_exp(ll[, t] + lw - .log_sum_exp(lw))
    }
    elpd_lfo <- sum(elpds)
    elpd_se <- sqrt(length(elpds)) * stats::sd(elpds)
  }
  list(waic = w, elpd_lfo = elpd_lfo, elpd_se = elpd_se,
       pareto_k = pareto_k, refresh_times = refreshes)
}

.extract_log_lik <- function(fit) {
  sf <- fit$stanfit
  pars <- sf@model_pars
  if (!"log_lik" %in% pars) return(NULL)
  as.matrix(rstan::extract(sf, pars = "log_lik", permuted = TRUE)$log_lik)
}

.log_weights_lfo <- function(log_lik, t, L) {
  if (t - L <= 1L) return(numeric(nrow(log_lik)))
  rowSums(log_lik[, (L + 1L):(t - 1L), drop = FALSE])
}

.pareto_k_fast <- function(lw) {
  w <- exp(lw - max(lw))
  n <- length(w)
  tail_n <- max(5L, floor(n / 5L))
  s <- sort(w, decreasing = TRUE)[seq_len(tail_n)]
  if (any(s <= 0)) return(NA_real_)
  .gpd_shape(s)
}

.gpd_shape <- function(x) {
  m <- length(x)
  if (m < 5L) return(NA_real_)
  x <- sort(x)
  prior <- 3
  n_grid <- 30L
  bs <- 1 / x[m] + (1 - sqrt(m / (seq_len(n_grid) - 0.5))) /
    (prior * x[floor(m / 4 + 0.5)])
  ks <- sapply(bs, function(b) {
    k <- mean(log1p(-b * x))
    m * (log(-b / k) - k - 1)
  })
  w <- 1 / sapply(bs, function(b) {
    sum(exp(ks - max(ks)) * (1 / (1e-9 + abs(b - bs))))
  })
  b_hat <- sum(bs * w) / sum(w)
  k_hat <- mean(log1p(-b_hat * x))
  k_hat
}

.log_mean_exp <- function(x) {
  m <- max(x)
  m + log(mean(exp(x - m)))
}

.log_sum_exp <- function(x) {
  m <- max(x)
  m + log(sum(exp(x - m)))
}

#' Predictive Stacking of Bayesian Models
#'
#' Computes stacking weights (Yao, Vehtari, Simpson, Gelman 2018) across a
#' list of fitted models by LOO-log-score maximization.
#'
#' @param fits Named list of \code{htdv_fit} objects.
#' @param method Stacking objective. Only \code{"log_score"} supported in this
#'   release.
#'
#' @return A list with \code{weights}, \code{model_names}, and
#'   \code{elpd_per_model}.
#'
#' @examples
#' \donttest{
#' if (requireNamespace("loo", quietly = TRUE)) {
#'   x <- rnorm(50)
#'   f1 <- htdv_fit(x, model = "tac", chains = 2, iter = 500,
#'                  refresh = 0, seed = 1)
#'   f2 <- htdv_fit(x, model = "wsc", chains = 2, iter = 500,
#'                  refresh = 0, seed = 2)
#'   htdv_stack(list(tac = f1, wsc = f2))$weights
#' }
#' }
#'
#' @export
htdv_stack <- function(fits, method = c("log_score")) {
  method <- match.arg(method)
  .require_suggested("loo")
  if (!is.list(fits) || length(fits) < 2L) {
    stop("'fits' must be a list with at least two 'htdv_fit' objects.",
         call. = FALSE)
  }
  ll_list <- lapply(fits, .extract_log_lik)
  if (any(vapply(ll_list, is.null, logical(1L)))) {
    stop("All fits must contain 'log_lik'.", call. = FALSE)
  }
  ncols <- vapply(ll_list, ncol, integer(1L))
  if (length(unique(ncols)) != 1L) {
    stop("All fits must have 'log_lik' with identical number of observations; ",
         "got ", paste(ncols, collapse = ", "),
         ". Whittle and time-domain likelihoods cannot be stacked directly.",
         call. = FALSE)
  }
  loo_list <- lapply(ll_list, function(ll) loo::loo(ll, cores = 1L))
  lpd_point <- vapply(loo_list,
                      function(l) as.numeric(l$pointwise[, "elpd_loo"]),
                      numeric(ncols[[1L]]))
  if (!is.matrix(lpd_point)) lpd_point <- as.matrix(lpd_point)
  w <- .stacking_weights(lpd_point)
  list(weights = stats::setNames(w, names(fits)),
       model_names = names(fits),
       elpd_per_model = vapply(loo_list,
                               function(l) l$estimates["elpd_loo", 1L],
                               numeric(1L)))
}

.stacking_weights <- function(lpd) {
  K <- ncol(lpd)
  n <- nrow(lpd)
  neg_obj <- function(z) {
    w <- .simplex(z)
    -sum(log(pmax(lpd %*% w, .Machine$double.eps)))
  }
  start <- rep(0, K - 1L)
  opt <- stats::optim(start, neg_obj, method = "BFGS")
  .simplex(opt$par)
}

.simplex <- function(z) {
  e <- c(exp(z), 1)
  e / sum(e)
}

test_that("htdv_rope accepts null when draws concentrate inside ROPE", {
  set.seed(401)
  d <- stats::rnorm(4000, mean = 0, sd = 0.01)
  out <- htdv_rope(d, rope = c(-0.1, 0.1))
  expect_equal(out$decision, "accept")
})

test_that("htdv_rope rejects when draws are far from ROPE", {
  set.seed(402)
  d <- stats::rnorm(4000, mean = 2, sd = 0.1)
  out <- htdv_rope(d, rope = c(-0.1, 0.1))
  expect_equal(out$decision, "reject")
})

test_that("htdv_rope returns undecided for borderline draws", {
  set.seed(403)
  d <- stats::rnorm(4000, mean = 0, sd = 0.25)
  out <- htdv_rope(d, rope = c(-0.1, 0.1))
  expect_true(out$decision %in% c("accept", "reject", "undecided"))
})

test_that("htdv_rope validates ROPE argument", {
  expect_error(htdv_rope(stats::rnorm(200), rope = c(1, 0)))
  expect_error(htdv_rope(stats::rnorm(200), rope = 1))
})

#' Whittle Likelihood for Stationary Series
#'
#' Computes the Whittle pseudo-log-likelihood at Fourier frequencies for a
#' user-supplied spectral density.
#'
#' @param x Numeric vector.
#' @param spec_fun Function \code{function(lambda, theta)} returning spectral
#'   density values at angular frequencies \code{lambda}.
#' @param theta Numeric parameter vector passed to \code{spec_fun}.
#' @param taper Logical; if \code{TRUE}, a 10-percent cosine taper is applied.
#'
#' @return A list with \code{loglik} (scalar), \code{frequencies},
#'   \code{periodogram}, and \code{fitted_spectrum}.
#'
#' @references
#' Whittle, P. (1953). Estimation and Information in Stationary Time Series.
#' Arkiv foer Matematik, 2(5), 423-434.
#' Dzhaparidze, K. (1986). Parameter Estimation and Hypothesis Testing in
#' Spectral Analysis of Stationary Time Series. Springer.
#'
#' @examples
#' x <- arima.sim(model = list(ar = 0.5), n = 200, rand.gen = rnorm)
#' ar1_spec <- function(lambda, theta) {
#'   phi <- theta[1]; sigma2 <- theta[2]
#'   sigma2 / (2 * pi * (1 - 2 * phi * cos(lambda) + phi^2))
#' }
#' htdv_whittle(as.numeric(x), ar1_spec, c(0.5, 1))$loglik
#'
#' @export
htdv_whittle <- function(x, spec_fun, theta, taper = FALSE) {
  .assert_numeric_vector(x, "x")
  if (!is.function(spec_fun)) {
    stop("'spec_fun' must be a function of (lambda, theta).", call. = FALSE)
  }
  n <- length(x)
  u <- .demean(x)
  if (isTRUE(taper)) u <- .cosine_taper(u, p = 0.1)
  pg <- .periodogram(u)
  lambda <- pg$lambda
  inp <- pg$intensity
  fv <- spec_fun(lambda, theta)
  if (any(!is.finite(fv)) || any(fv <= 0)) {
    return(list(loglik = -Inf, frequencies = lambda, periodogram = inp,
                fitted_spectrum = fv))
  }
  loglik <- -sum(log(fv) + inp / fv)
  list(loglik = loglik, frequencies = lambda, periodogram = inp,
       fitted_spectrum = fv)
}

.periodogram <- function(u) {
  n <- length(u)
  m <- floor(n / 2)
  j <- seq_len(m)
  lambda <- 2 * pi * j / n
  fft_u <- stats::fft(u)
  inp <- (1 / (2 * pi * n)) * Mod(fft_u[j + 1L])^2
  list(lambda = lambda, intensity = inp)
}

.cosine_taper <- function(u, p = 0.1) {
  n <- length(u)
  m <- floor(p * n / 2)
  if (m < 1) return(u)
  w <- rep(1, n)
  seq_edge <- seq_len(m)
  w[seq_edge] <- 0.5 * (1 - cos(pi * (seq_edge - 0.5) / m))
  w[(n - m + 1):n] <- rev(w[seq_edge])
  u * w
}

test_that("htdv_whittle returns finite log-likelihood for AR(1)", {
  set.seed(501)
  x <- stats::arima.sim(model = list(ar = 0.5), n = 200,
                        rand.gen = stats::rnorm)
  ar1_spec <- function(lambda, theta) {
    phi <- theta[1]
    sigma2 <- theta[2]
    sigma2 / (2 * pi * (1 - 2 * phi * cos(lambda) + phi^2))
  }
  out <- htdv_whittle(as.numeric(x), ar1_spec, c(0.5, 1))
  expect_true(is.finite(out$loglik))
  expect_equal(length(out$frequencies), length(out$periodogram))
})

test_that("htdv_whittle rejects non-positive spectrum", {
  x <- stats::rnorm(50)
  bad_spec <- function(lambda, theta) rep(-1, length(lambda))
  out <- htdv_whittle(x, bad_spec, theta = 1)
  expect_true(is.infinite(out$loglik))
})

test_that("htdv_composite evaluates pairwise Gaussian correctly", {
  set.seed(502)
  x <- stats::arima.sim(model = list(ar = 0.3), n = 150,
                        rand.gen = stats::rnorm)
  gauss_ar1 <- function(block, theta) {
    phi <- theta[1]
    sigma2 <- theta[2]
    a <- block[1]
    b <- block[2]
    stats::dnorm(a, 0, sqrt(sigma2 / (1 - phi^2)), log = TRUE) +
      stats::dnorm(b, phi * a, sqrt(sigma2), log = TRUE)
  }
  out <- htdv_composite(as.numeric(x), gauss_ar1, c(0.3, 1),
                        k = 1L, log = TRUE)
  expect_true(is.finite(out$loglik))
  expect_equal(out$n_blocks, length(x) - 1L)
})

#' HTDV: Hypothesis Testing for Dependent Variables with Unbalanced Data
#'
#' Implements hierarchical Bayesian inference, robust frequentist inference,
#' and distribution-free inference for dependent and unbalanced data under
#' strong-mixing conditions.
#'
#' @section Main functions:
#' \itemize{
#'   \item \code{\link{htdv_fit}}: unified hierarchical Bayesian fit via HMC.
#'   \item \code{\link{htdv_tac}}, \code{\link{htdv_wsc}}, \code{\link{htdv_mpc}}:
#'         convergence-regime-specific fits.
#'   \item \code{\link{htdv_whittle}}, \code{\link{htdv_composite}}:
#'         likelihood surrogates for dependent data.
#'   \item \code{\link{htdv_envelope}}: Berger-robust posterior envelope.
#'   \item \code{\link{htdv_lrv}}: long-run variance with Newey-West and
#'         Andrews bandwidth.
#'   \item \code{\link{htdv_fixedb}}: fixed-bandwidth HAR inference
#'         (Kiefer-Vogelsang 2005).
#'   \item \code{\link{htdv_boot}}: block bootstrap with Patton-Politis-White
#'         automatic block length.
#'   \item \code{\link{htdv_conformal}}: adaptive conformal inference
#'         (Gibbs-Candes 2021).
#'   \item \code{\link{htdv_rope}}: posterior decision under Region of
#'         Practical Equivalence (Kruschke 2018).
#'   \item \code{\link{htdv_bf}}: bridge-sampling Bayes factor.
#'   \item \code{\link{htdv_waic_lfo}}: WAIC and leave-future-out
#'         cross-validation.
#'   \item \code{\link{htdv_stack}}: predictive stacking across models.
#'   \item \code{\link{htdv_diagnostics}}, \code{\link{htdv_ppc}}:
#'         MCMC diagnostics and posterior-predictive checks.
#'   \item \code{\link{htdv_equivalence_constants}}: explicit numerical
#'         constants for TAC/WSC/MPC metric equivalence.
#' }
#'
#' @docType package
#' @name HTDV-package
#' @aliases HTDV
#' @keywords internal
"_PACKAGE"

#' Adaptive Conformal Inference for Dependent Data
#'
#' Online adaptive conformal prediction intervals with gradient update on the
#' miscoverage rate (Gibbs-Candes 2021). Provides long-run marginal coverage
#' without exchangeability.
#'
#' @param x Numeric vector of sequential observations.
#' @param predictor Function \code{function(history)} returning a scalar
#'   one-step point prediction.
#' @param residual_fun Function of residuals to calibrate the score; default
#'   absolute residual.
#' @param alpha_target Target miscoverage rate.
#' @param lambda Step size for the gradient update.
#' @param burn_in Integer; the first \code{burn_in} observations are used only
#'   to warm up the predictor.
#'
#' @return A list with \code{intervals} (matrix of lower/upper),
#'   \code{coverage} (empirical), \code{alpha_path}, and
#'   \code{point_predictions}.
#'
#' @references
#' Gibbs, I., & Candes, E. (2021). Adaptive Conformal Inference under
#' Distribution Shift. NeurIPS 34: 1660-1672.
#'
#' @examples
#' x <- arima.sim(model = list(ar = 0.6), n = 200, rand.gen = rnorm)
#' pred <- function(hist) if (length(hist) >= 1) hist[length(hist)] else 0
#' out <- htdv_conformal(as.numeric(x), pred, alpha_target = 0.1,
#'                       lambda = 0.05, burn_in = 20)
#' out$coverage
#'
#' @export
htdv_conformal <- function(x, predictor,
                           residual_fun = function(e) abs(e),
                           alpha_target = 0.1,
                           lambda = 0.05,
                           burn_in = 20L) {
  .assert_numeric_vector(x, "x")
  if (!is.function(predictor)) {
    stop("'predictor' must be a function of the history vector.",
         call. = FALSE)
  }
  .assert_probability(alpha_target, "alpha_target")
  if (!is.numeric(lambda) || length(lambda) != 1L || lambda <= 0 || lambda >= 1) {
    stop("'lambda' must be a scalar in (0, 1).", call. = FALSE)
  }
  n <- length(x)
  if (burn_in < 1L || burn_in >= n) {
    stop("'burn_in' must be an integer in [1, length(x)-1].", call. = FALSE)
  }
  alpha_t <- alpha_target
  scores <- numeric(0L)
  yhat <- numeric(n)
  intervals <- matrix(NA_real_, nrow = n, ncol = 2L)
  alpha_path <- numeric(n)
  colnames(intervals) <- c("lower", "upper")
  for (t in seq_len(n)) {
    hist_t <- x[seq_len(t - 1L)]
    yhat[t] <- if (t > 1L) predictor(hist_t) else 0
    alpha_path[t] <- alpha_t
    if (t > burn_in && length(scores) > 0L) {
      q <- as.numeric(stats::quantile(scores,
                                      probs = min(1, max(0, 1 - alpha_t)),
                                      names = FALSE, na.rm = TRUE))
      intervals[t, ] <- c(yhat[t] - q, yhat[t] + q)
      covered <- (x[t] >= intervals[t, 1L]) && (x[t] <= intervals[t, 2L])
      alpha_t <- alpha_t + lambda * (alpha_target - (1 - as.integer(covered)))
      alpha_t <- min(0.99, max(0.01, alpha_t))
    }
    if (t > 1L) {
      scores <- c(scores, as.numeric(residual_fun(x[t] - yhat[t])))
    }
  }
  valid <- !is.na(intervals[, 1L])
  cov_emp <- if (any(valid)) {
    mean((x[valid] >= intervals[valid, 1L]) &
         (x[valid] <= intervals[valid, 2L]))
  } else NA_real_
  list(intervals = intervals, coverage = cov_emp,
       alpha_path = alpha_path, point_predictions = yhat,
       alpha_target = alpha_target, lambda = lambda)
}

test_that("htdv_conformal attains approximate target coverage", {
  set.seed(301)
  x <- stats::arima.sim(model = list(ar = 0.4), n = 300,
                        rand.gen = stats::rnorm)
  pred <- function(history) {
    if (length(history) >= 1L) history[length(history)] else 0
  }
  out <- htdv_conformal(as.numeric(x), pred,
                        alpha_target = 0.1,
                        lambda = 0.05,
                        burn_in = 30L)
  expect_true(is.numeric(out$coverage))
  expect_true(out$coverage >= 0.5 && out$coverage <= 1)
})

test_that("htdv_conformal validates arguments", {
  expect_error(htdv_conformal(stats::rnorm(10), mean, alpha_target = 0))
  expect_error(htdv_conformal(stats::rnorm(10), mean, lambda = -1))
  expect_error(htdv_conformal(stats::rnorm(10), mean, burn_in = 0))
})

#' Aggregated Output of the HTDV Factorial Simulation Study
#'
#' Sign-corrected aggregated results of the factorial Monte Carlo study
#' described in Section 12-bis of the companion paper. The study crosses
#' five factors at four levels each (sample size, AR(1) coefficient,
#' innovation tail, imbalance ratio, location shift) for 1024 cells, each
#' replicated 500 times against three inferential layers (HAR, bootstrap,
#' hierarchical Bayesian). One cell (n=500, phi=0.85, t3, imb=1, delta=0)
#' was lost to an I/O collision and is excluded; the table contains
#' 1023 cells x 3 layers = 3069 rows.
#'
#' @format A data frame with one row per (cell, layer) and the following
#' columns:
#' \describe{
#'   \item{n}{Per-group primary sample size (40, 80, 200, 500).}
#'   \item{phi}{AR(1) coefficient (0, 0.3, 0.6, 0.85).}
#'   \item{tail}{Innovation distribution: \code{"normal"}, \code{"t5"},
#'     \code{"t3"}, \code{"t2_1"}.}
#'   \item{imb}{Imbalance ratio n1/n2 (1, 1.5, 3, 6).}
#'   \item{delta}{Location shift in units of sigma_inf (0, 0.1, 0.25, 0.5).}
#'   \item{layer}{Inferential layer: \code{"har"}, \code{"boot"},
#'     \code{"bayes"}.}
#'   \item{n_reps}{Number of replications aggregated (500 for all cells).}
#'   \item{reject_rate}{Empirical rejection rate at nominal alpha = 0.05.}
#'   \item{coverage}{Fraction of replications whose 95\% interval covered
#'     the true parameter (sign-corrected for HAR and bootstrap).}
#'   \item{mean_ci_length}{Average length of the 95\% interval.}
#'   \item{mean_runtime}{Mean per-replication runtime in seconds.}
#'   \item{diag_pass_rate}{Fraction of replications passing all HMC
#'     diagnostic gates (Bayesian layer only; \code{NA} otherwise).}
#' }
#'
#' @source Generated by \code{run_simstudy.R} of the companion paper
#'   repository on master seed \code{20260422}, with sign-corrected
#'   coverage produced by \code{recover_coverage.R}. Total wall-clock
#'   31.1 hours on a 16-core Linux workstation. See
#'   \code{vignette("HTDV-validation")} for the full report.
"htdv_sim_summary"

#' HTDV Empirical Benchmarks Against Published References
#'
#' Output of the three-dataset external validation protocol of
#' Section 12-ter of the companion paper. Each dataset is run against a
#' pre-existing reference result from the published literature on the
#' same vintage, with the three HTDV inferential layers (HAR, bootstrap,
#' hierarchical Bayesian) reporting their point estimate, 95\% interval,
#' and a Bayesian diagnostic-pass flag.
#'
#' @format A data frame with three rows (one per dataset) and the
#' following columns:
#' \describe{
#'   \item{dataset}{Name of the benchmark dataset.}
#'   \item{reference}{Citation of the reference paper.}
#'   \item{n}{Sample size after preprocessing.}
#'   \item{sample}{Sample window (string).}
#'   \item{reference_value}{Reference point value.}
#'   \item{reference_se}{Reference standard error (\code{NA} when the
#'     reference does not provide one).}
#'   \item{har_point, har_ci_lo, har_ci_hi}{HAR-Wald point and 95\% CI.}
#'   \item{bayes_point, bayes_ci_lo, bayes_ci_hi}{Bayesian point and
#'     95\% credible interval.}
#'   \item{bayes_diag_pass, bayes_rhat_max, bayes_ess_min,
#'     bayes_divergences}{Bayesian HMC diagnostics.}
#'   \item{boot_point, boot_ci_lo, boot_ci_hi}{Stationary block bootstrap
#'     point and 95\% percentile CI.}
#'   \item{agreement_har, agreement_bayes, agreement_boot}{Per-layer
#'     agreement verdict: \code{"agreement"}, \code{"htdv-strict (above ref)"},
#'     or \code{"htdv-strict (below ref)"}.}
#' }
#'
#' @details
#' The three benchmarks (E1, E2, E3) and their references are:
#' \itemize{
#'   \item{\strong{E1.} Post-1984 US CPI inflation against Stock and
#'     Watson (2007). FRED-MD vintage 2026-03, variable \code{CPIAUCSL}
#'     transformed to annualized monthly inflation.}
#'   \item{\strong{E2.} Long-run mean of log-CAPE against Campbell and
#'     Shiller (1998). Robert Shiller's online historical dataset.}
#'   \item{\strong{E3.} US vs. Canada 10-year government bond yield
#'     differential against the iid Welch baseline. FRED series
#'     \code{GS10} and \code{IRLTLT01CAM156N}, common sample 1990-01 to
#'     latest.}
#' }
#'
#' @source Generated by \code{run_benchmarks.R} of the companion paper
#'   repository. See \code{vignette("HTDV-validation")} for the
#'   methodological narrative and the persistence-ladder reading.
"htdv_empirical_benchmarks"

#' Hierarchical Bayesian Fit for Dependent Variables
#'
#' Fits the hierarchical Bayesian model described in the companion paper via
#' Hamiltonian Monte Carlo (NUTS) using \pkg{rstan}. Nuisance parameters
#' \code{sigma_inf}, \code{gamma_mix}, and (for WSC) \code{b} carry priors and
#' are integrated over.
#'
#' @param x Numeric vector of observations.
#' @param model One of \code{"tac"}, \code{"wsc"}, \code{"mpc"},
#'   \code{"whittle"}, \code{"composite"}.
#' @param chains Number of HMC chains.
#' @param iter Total iterations per chain (warmup plus sampling).
#' @param warmup Number of warmup iterations.
#' @param refresh Stan progress refresh rate (0 suppresses output).
#' @param adapt_delta HMC adaptation target acceptance.
#' @param max_treedepth Maximum NUTS treedepth.
#' @param seed Optional integer seed forwarded to \pkg{rstan}.
#' @param ... Additional arguments forwarded to \code{rstan::sampling}.
#'
#' @return An object of class \code{htdv_fit}, a list with components
#'   \code{stanfit}, \code{model}, \code{data}, \code{n}, \code{call}.
#'
#' @references
#' Hoffman, M.D., & Gelman, A. (2014). The No-U-Turn Sampler. JMLR 15(1):
#' 1593-1623.
#' Betancourt, M. (2016). Diagnosing Suboptimal Cotangent Disintegrations in
#' HMC. <doi:10.48550/arXiv.1604.00695>.
#'
#' @examples
#' \donttest{
#' x <- rnorm(50)
#' fit <- htdv_fit(x, model = "tac", chains = 2, iter = 500,
#'                 refresh = 0, seed = 1)
#' class(fit)
#' }
#'
#' @export
htdv_fit <- function(x,
                     model = c("tac", "wsc", "mpc", "whittle", "composite"),
                     chains = 4L, iter = 4000L, warmup = iter %/% 2L,
                     refresh = max(iter %/% 10L, 1L),
                     adapt_delta = 0.99, max_treedepth = 12L,
                     seed = NULL, ...) {
  model <- match.arg(model)
  .assert_numeric_vector(x, "x")
  stan_name <- switch(model,
    tac       = "tac_hierarchical",
    wsc       = "wsc_hierarchical",
    mpc       = "mpc_hierarchical",
    whittle   = "whittle_hierarchical",
    composite = "composite_hierarchical"
  )
  sm <- .load_stan_model(stan_name)
  stan_data <- .build_stan_data(x = x, model = model)
  control <- list(adapt_delta = adapt_delta, max_treedepth = max_treedepth)
  args <- list(object = sm, data = stan_data, chains = chains,
               iter = iter, warmup = warmup, refresh = refresh,
               control = control, ...)
  if (!is.null(seed)) args$seed <- seed
  stanfit <- do.call(rstan::sampling, args)
  out <- list(stanfit = stanfit, model = model, data = stan_data,
              n = length(x), call = match.call())
  class(out) <- "htdv_fit"
  out
}

.build_stan_data <- function(x, model) {
  n <- length(x)
  lrv <- htdv_lrv(x, kernel = "qs", bandwidth = "andrews")$lrv
  base <- list(N = n, x = as.numeric(x),
               sigma_inf_scale = max(2 * sqrt(lrv), .Machine$double.eps),
               gamma_shape = 2, gamma_rate = 1,
               mu0 = mean(x),
               tau0 = max(10 * stats::sd(x), .Machine$double.eps))
  if (model == "wsc") {
    base$b_lower <- max(2L, floor(0.5 * sqrt(n)))
    base$b_upper <- max(base$b_lower + 1L, ceiling(2 * sqrt(n)))
  }
  base
}

#' @rdname htdv_fit
#' @export
htdv_tac <- function(x, ...) htdv_fit(x = x, model = "tac", ...)

#' @rdname htdv_fit
#' @export
htdv_wsc <- function(x, ...) htdv_fit(x = x, model = "wsc", ...)

#' @rdname htdv_fit
#' @export
htdv_mpc <- function(x, ...) htdv_fit(x = x, model = "mpc", ...)

#' Berger-Robust Envelope Posterior
#'
#' Builds an envelope posterior over a list of \code{htdv_fit} objects by
#' entropy-maximizing mixture weights subject to predictive consistency
#' (Berger 1994).
#'
#' @param fits List of \code{htdv_fit} objects on the same data.
#' @param target Name of the Stan parameter over which to build the envelope.
#'
#' @return A list with \code{draws} (envelope draws for the target parameter),
#'   \code{weights}, \code{intervals}, and \code{component_intervals}.
#'
#' @references
#' Berger, J.O. (1994). An Overview of Robust Bayesian Analysis. Test 3(1):
#' 5-124.
#'
#' @examples
#' \donttest{
#' x <- rnorm(50)
#' f1 <- htdv_fit(x, model = "tac", chains = 2, iter = 500,
#'                refresh = 0, seed = 1)
#' f2 <- htdv_fit(x, model = "wsc", chains = 2, iter = 500,
#'                refresh = 0, seed = 2)
#' htdv_envelope(list(f1, f2), target = "theta")$intervals
#' }
#'
#' @export
htdv_envelope <- function(fits, target = "theta") {
  if (!is.list(fits) || !all(vapply(fits, inherits, logical(1L), "htdv_fit"))) {
    stop("'fits' must be a list of 'htdv_fit' objects.", call. = FALSE)
  }
  draws_list <- lapply(fits, function(f) {
    d <- rstan::extract(f$stanfit, pars = target, permuted = TRUE)[[1L]]
    as.numeric(d)
  })
  lengths <- vapply(draws_list, length, integer(1L))
  if (length(unique(lengths)) > 1L) {
    lo <- min(lengths)
    draws_list <- lapply(draws_list, function(d) d[seq_len(lo)])
  }
  ent <- vapply(draws_list, function(d) {
    sd_d <- stats::sd(d)
    if (!is.finite(sd_d) || sd_d <= 0) 0 else log(sd_d)
  }, numeric(1L))
  w <- exp(ent - max(ent))
  w <- w / sum(w)
  k <- length(draws_list)
  n_env <- min(vapply(draws_list, length, integer(1L)))
  pick <- sample.int(k, size = n_env, replace = TRUE, prob = w)
  envelope <- vapply(seq_len(n_env), function(i) draws_list[[pick[i]]][i],
                     numeric(1L))
  comp_ci <- t(vapply(draws_list, function(d) {
    as.numeric(stats::quantile(d, probs = c(0.025, 0.975), names = FALSE))
  }, numeric(2L)))
  env_ci <- as.numeric(stats::quantile(envelope, probs = c(0.025, 0.975),
                                       names = FALSE))
  list(draws = envelope, weights = w,
       intervals = env_ci, component_intervals = comp_ci)
}

#' @export
print.htdv_fit <- function(x, ...) {
  cat("HTDV fit\n")
  cat("  model:     ", x$model, "\n", sep = "")
  cat("  n:         ", x$n, "\n", sep = "")
  cat("  chains:    ", x$stanfit@sim$chains, "\n", sep = "")
  cat("  iters:     ", x$stanfit@sim$iter, "\n", sep = "")
  invisible(x)
}

#' @export
summary.htdv_fit <- function(object, pars = NULL, probs = c(0.025, 0.5, 0.975),
                             ...) {
  s <- if (is.null(pars)) {
    rstan::summary(object$stanfit, probs = probs)
  } else {
    rstan::summary(object$stanfit, pars = pars, probs = probs)
  }
  s$summary
}

#' @export
plot.htdv_fit <- function(x, pars = NULL, ...) {
  if (is.null(pars)) pars <- "theta"
  rstan::traceplot(x$stanfit, pars = pars, ...)
}

#' Print-Friendly Summary for an HTDV Fit
#'
#' @param fit An \code{htdv_fit} object.
#' @param rope Optional two-element vector for ROPE decision on the target.
#' @param target Parameter name to summarize.
#'
#' @return A data frame with posterior mean, sd, 2.5\%, 50\%, 97.5\%, Rhat,
#'   bulk and tail ESS for \code{target}, plus the ROPE decision if requested.
#'
#' @examples
#' \donttest{
#' x <- rnorm(50)
#' fit <- htdv_fit(x, model = "tac", chains = 2, iter = 500,
#'                 refresh = 0, seed = 1)
#' htdv_summary(fit, target = "theta")
#' }
#'
#' @export
htdv_summary <- function(fit, rope = NULL, target = "theta") {
  if (!inherits(fit, "htdv_fit")) {
    stop("'fit' must be an 'htdv_fit' object.", call. = FALSE)
  }
  s <- rstan::summary(fit$stanfit, pars = target,
                      probs = c(0.025, 0.5, 0.975))$summary
  df <- as.data.frame(s)
  if (!is.null(rope)) {
    draws <- as.numeric(rstan::extract(fit$stanfit, pars = target,
                                       permuted = TRUE)[[1L]])
    dec <- htdv_rope(draws, rope = rope)
    df$rope_decision <- dec$decision
  }
  df
}

test_that("htdv_fit returns an htdv_fit object for a minimal TAC run", {
  skip_on_cran()
  skip_if_not_installed("rstan")
  set.seed(601)
  x <- stats::rnorm(40)
  fit <- suppressWarnings(
    htdv_fit(x, model = "tac", chains = 1L, iter = 200L,
             warmup = 100L, refresh = 0L, seed = 1L))
  expect_s3_class(fit, "htdv_fit")
  expect_equal(fit$model, "tac")
  expect_equal(fit$n, length(x))
})

test_that("htdv_envelope combines draws across models", {
  skip_on_cran()
  skip_if_not_installed("rstan")
  set.seed(602)
  x <- stats::rnorm(40)
  f1 <- suppressWarnings(
    htdv_fit(x, model = "tac", chains = 1L, iter = 200L,
             warmup = 100L, refresh = 0L, seed = 2L))
  f2 <- suppressWarnings(
    htdv_fit(x, model = "composite", chains = 1L, iter = 200L,
             warmup = 100L, refresh = 0L, seed = 3L))
  env <- htdv_envelope(list(f1, f2), target = "theta")
  expect_true(length(env$draws) > 0L)
  expect_length(env$intervals, 2L)
})

#' Fixed-Bandwidth HAR Inference (Kiefer-Vogelsang 2005)
#'
#' Fixed-b HAR Wald test where the bandwidth is a fixed fraction of the sample
#' size. Critical values are obtained by simulation from the asymptotic
#' functional distribution.
#'
#' @param x Numeric vector.
#' @param theta0 Null value for the mean (default 0).
#' @param B Fixed bandwidth fraction in (0, 1].
#' @param kernel Kernel choice (see \code{\link{htdv_lrv}}).
#' @param sims Number of Monte Carlo draws for the critical-value simulation.
#' @param alpha Significance level.
#'
#' @return A list with \code{statistic}, \code{critical_value}, \code{p_value},
#'   \code{reject}, \code{bandwidth}, and \code{kernel}.
#'
#' @references
#' Kiefer, N.M., & Vogelsang, T.J. (2005). A New Asymptotic Theory for
#' Heteroskedasticity-Autocorrelation Robust Tests. Econometric Theory
#' 21(6): 1130-1164.
#'
#' @examples
#' set.seed(1)
#' x <- arima.sim(model = list(ar = 0.5), n = 200, rand.gen = rnorm)
#' htdv_fixedb(as.numeric(x), theta0 = 0, B = 0.2, sims = 1000)$p_value
#'
#' @export
htdv_fixedb <- function(x, theta0 = 0,
                        B = 0.1,
                        kernel = c("bartlett", "parzen", "qs"),
                        sims = 5000L, alpha = 0.05) {
  kernel <- match.arg(kernel)
  .assert_numeric_vector(x, "x")
  if (!is.numeric(B) || length(B) != 1L || B <= 0 || B > 1) {
    stop("'B' must be a scalar in (0, 1].", call. = FALSE)
  }
  .assert_probability(alpha, "alpha")
  n <- length(x)
  b <- max(floor(B * n), 1L)
  u <- x - theta0
  mu_hat <- mean(x)
  resid <- x - mu_hat
  lrv <- .kernel_sum(.auto_cov(resid, lag_max = n - 1L),
                     h = b, kernel = kernel, n = n)
  stat <- n * (mu_hat - theta0)^2 / lrv
  null_draws <- .simulate_fixedb_null(n = n, B = B, kernel = kernel,
                                      sims = sims)
  cv <- as.numeric(stats::quantile(null_draws, probs = 1 - alpha,
                                   names = FALSE))
  pv <- mean(null_draws >= stat)
  list(statistic = stat, critical_value = cv, p_value = pv,
       reject = stat > cv, bandwidth = b, kernel = kernel)
}

.simulate_fixedb_null <- function(n, B, kernel, sims) {
  b <- max(floor(B * n), 1L)
  out <- numeric(sims)
  for (s in seq_len(sims)) {
    z <- stats::rnorm(n)
    mu <- mean(z)
    r <- z - mu
    lrv <- .kernel_sum(.auto_cov(r, lag_max = n - 1L),
                       h = b, kernel = kernel, n = n)
    out[s] <- n * mu^2 / lrv
  }
  out
}

test_that("htdv_fixedb returns a valid p-value on iid data", {
  set.seed(101)
  x <- stats::rnorm(150)
  out <- htdv_fixedb(x, theta0 = 0, B = 0.1,
                     sims = 500L)
  expect_true(out$p_value >= 0 && out$p_value <= 1)
  expect_true(is.finite(out$statistic))
})

test_that("htdv_fixedb tends to reject when H0 is violated", {
  set.seed(102)
  x <- stats::rnorm(200, mean = 2)
  out <- htdv_fixedb(x, theta0 = 0, B = 0.1,
                     sims = 500L)
  expect_true(out$reject)
})

test_that("htdv_fixedb checks argument validity", {
  expect_error(htdv_fixedb(stats::rnorm(50), B = 0))
  expect_error(htdv_fixedb(stats::rnorm(50), B = 1.5))
  expect_error(htdv_fixedb(stats::rnorm(50), alpha = -0.1))
})

#' Block Bootstrap with Automatic Block Length
#'
#' Implements circular and stationary block bootstraps (Politis-Romano 1992,
#' 1994) with Patton-Politis-White (2009) automatic block length selection.
#'
#' @param x Numeric vector.
#' @param statistic Function \code{function(x)} returning a scalar or numeric
#'   vector.
#' @param R Number of bootstrap replicates.
#' @param type \code{"circular"} or \code{"stationary"}.
#' @param block_length Either \code{"auto"} (Patton-Politis-White) or a
#'   positive integer.
#' @param level Confidence level for intervals.
#'
#' @return A list with \code{t0} (statistic on original data), \code{t_star}
#'   (bootstrap distribution), \code{ci_percentile}, \code{ci_basic},
#'   \code{ci_studentized}, and \code{block_length}.
#'
#' @references
#' Politis, D.N., & Romano, J.P. (1994). The Stationary Bootstrap. JASA
#' 89(428): 1303-1313.
#' Patton, A., Politis, D.N., & White, H. (2009). Correction to Automatic
#' Block-Length Selection for the Dependent Bootstrap. Econometric Reviews
#' 28(4): 372-375.
#'
#' @examples
#' set.seed(1)
#' x <- arima.sim(model = list(ar = 0.5), n = 200, rand.gen = rnorm)
#' out <- htdv_boot(as.numeric(x), mean, R = 500, type = "stationary")
#' out$ci_percentile
#'
#' @export
htdv_boot <- function(x, statistic, R = 1999L,
                      type = c("circular", "stationary"),
                      block_length = "auto",
                      level = 0.95) {
  type <- match.arg(type)
  .assert_numeric_vector(x, "x")
  if (!is.function(statistic)) {
    stop("'statistic' must be a function of x.", call. = FALSE)
  }
  .assert_probability(level, "level")
  n <- length(x)
  b_opt <- if (identical(block_length, "auto")) {
    .ppw_block_length(x)
  } else {
    if (!is.numeric(block_length) || block_length < 1L ||
        block_length != floor(block_length)) {
      stop("'block_length' must be 'auto' or a positive integer.",
           call. = FALSE)
    }
    as.integer(block_length)
  }
  t0 <- statistic(x)
  t_star <- matrix(NA_real_, nrow = R, ncol = length(t0))
  for (r in seq_len(R)) {
    xb <- if (type == "circular") {
      .circular_bootstrap_sample(x, b = b_opt)
    } else {
      .stationary_bootstrap_sample(x, p = 1 / b_opt)
    }
    t_star[r, ] <- as.numeric(statistic(xb))
  }
  q_lo <- (1 - level) / 2
  q_hi <- 1 - q_lo
  ci_perc <- apply(t_star, 2L, stats::quantile,
                   probs = c(q_lo, q_hi), names = FALSE,
                   na.rm = TRUE)
  ci_basic <- rbind(2 * t0 - ci_perc[2L, ], 2 * t0 - ci_perc[1L, ])
  dimnames(ci_basic) <- dimnames(ci_perc)
  sd_star <- apply(t_star, 2L, stats::sd, na.rm = TRUE)
  z_lo <- stats::qnorm(q_lo)
  z_hi <- stats::qnorm(q_hi)
  ci_student <- rbind(t0 + z_lo * sd_star, t0 + z_hi * sd_star)
  dimnames(ci_student) <- dimnames(ci_perc)
  list(t0 = t0, t_star = t_star, ci_percentile = ci_perc,
       ci_basic = ci_basic, ci_studentized = ci_student,
       block_length = b_opt, type = type, level = level)
}

.circular_bootstrap_sample <- function(x, b) {
  n <- length(x)
  nb <- ceiling(n / b)
  starts <- sample.int(n, size = nb, replace = TRUE)
  idx <- integer(nb * b)
  for (i in seq_len(nb)) {
    s <- starts[i]
    block_idx <- ((s - 1L) + seq_len(b) - 1L) %% n + 1L
    idx[((i - 1L) * b + 1L):(i * b)] <- block_idx
  }
  x[idx[seq_len(n)]]
}

.stationary_bootstrap_sample <- function(x, p) {
  n <- length(x)
  out <- integer(n)
  out[1L] <- sample.int(n, size = 1L)
  for (t in 2:n) {
    if (stats::runif(1L) < p) {
      out[t] <- sample.int(n, size = 1L)
    } else {
      out[t] <- out[t - 1L] %% n + 1L
    }
  }
  x[out]
}

.ppw_block_length <- function(x) {
  n <- length(x)
  u <- .demean(x)
  max_lag <- floor(min(3 * sqrt(n), n / 3))
  acv <- .auto_cov(u, lag_max = max_lag)
  rho <- acv / acv[1L]
  thr <- 2 * sqrt(log10(n) / n)
  kn_m <- .find_significant_lag(rho[-1L], threshold = thr)
  m_hat <- min(2 * kn_m, max_lag)
  k_vec <- seq(-m_hat, m_hat)
  lam <- .flat_top_kernel(k_vec / m_hat)
  g_hat <- sum(lam * abs(k_vec) * acv[abs(k_vec) + 1L])
  d_hat <- 2 * (acv[1L] + 2 * sum(lam[k_vec > 0] * acv[k_vec[k_vec > 0] + 1L]))^2
  b <- max(1, round((2 * g_hat^2 / d_hat)^(1 / 3) * n^(1 / 3)))
  min(b, n %/% 2)
}

.find_significant_lag <- function(rho_nonzero, threshold) {
  K_limit <- max(5L, round(log10(length(rho_nonzero) + 1L)))
  for (k in seq_along(rho_nonzero)) {
    window_end <- min(k + K_limit, length(rho_nonzero))
    if (all(abs(rho_nonzero[k:window_end]) < threshold)) {
      return(k)
    }
  }
  length(rho_nonzero)
}

.flat_top_kernel <- function(z) {
  az <- abs(z)
  ifelse(az <= 0.5, 1,
         ifelse(az <= 1, 2 * (1 - az), 0))
}

test_that("htdv_boot percentile CI covers the true mean frequently", {
  set.seed(201)
  x <- stats::arima.sim(model = list(ar = 0.5), n = 150,
                        rand.gen = stats::rnorm)
  out <- htdv_boot(as.numeric(x), mean, R = 400, type = "stationary",
                   block_length = "auto")
  expect_true(nrow(out$ci_percentile) == 2)
  expect_true(is.finite(out$block_length))
  expect_true(out$block_length >= 1)
})

test_that("htdv_boot supports circular type", {
  set.seed(202)
  x <- stats::rnorm(100)
  out <- htdv_boot(x, mean, R = 200, type = "circular",
                   block_length = 5L)
  expect_true(nrow(out$ci_basic) == 2)
})

test_that("htdv_boot returns studentized CI matrix", {
  set.seed(203)
  x <- stats::rnorm(100)
  out <- htdv_boot(x, mean, R = 200)
  expect_true(nrow(out$ci_studentized) == 2)
})

#' Composite Log-Likelihood for Dependent Data
#'
#' Pairwise or \code{k}-tuple composite log-likelihood (Varin, Reid and Firth
#' 2011) for strictly stationary sequences.
#'
#' @param x Numeric vector.
#' @param density_fun Function \code{function(block, theta)} returning the
#'   joint density of a \code{(k+1)}-block of consecutive observations.
#' @param theta Numeric parameter vector.
#' @param k Window width (pairwise if \code{k = 1}).
#' @param log Logical; if \code{TRUE}, \code{density_fun} already returns log
#'   densities.
#'
#' @return A list with \code{loglik} (scalar), \code{n_blocks}, and
#'   \code{block_values}.
#'
#' @references
#' Varin, C., Reid, N., & Firth, D. (2011). An overview of composite
#' likelihood methods. Statistica Sinica, 21(1), 5-42.
#'
#' @examples
#' x <- arima.sim(model = list(ar = 0.4), n = 200, rand.gen = rnorm)
#' gauss_ar1 <- function(block, theta) {
#'   phi <- theta[1]; sigma2 <- theta[2]
#'   a <- block[1]; b <- block[2]
#'   dnorm(a, 0, sqrt(sigma2 / (1 - phi^2)), log = TRUE) +
#'     dnorm(b, phi * a, sqrt(sigma2), log = TRUE)
#' }
#' htdv_composite(as.numeric(x), gauss_ar1, c(0.4, 1), k = 1, log = TRUE)$loglik
#'
#' @export
htdv_composite <- function(x, density_fun, theta, k = 1L, log = FALSE) {
  .assert_numeric_vector(x, "x")
  if (!is.function(density_fun)) {
    stop("'density_fun' must be a function of (block, theta).", call. = FALSE)
  }
  if (!is.numeric(k) || length(k) != 1L || k < 1L || k != floor(k)) {
    stop("'k' must be a positive integer.", call. = FALSE)
  }
  n <- length(x)
  if (n < k + 1L) {
    stop("'x' is too short for the requested window 'k'.", call. = FALSE)
  }
  vals <- numeric(n - k)
  for (t in seq_len(n - k)) {
    block <- x[t:(t + k)]
    v <- density_fun(block, theta)
    vals[t] <- if (isTRUE(log)) v else log(max(v, .Machine$double.eps))
  }
  list(loglik = sum(vals), n_blocks = n - k, block_values = vals)
}

#' Numerical Constants for TAC/WSC/MPC Metric Equivalence
#'
#' Computes the explicit finite-sample constants K_TAC(gamma, q), K_MPC(gamma)
#' and the unified two-sided envelope (c_L, c_U) from Theorems 5-8 of the
#' companion paper.
#'
#' @param gamma Mixing decay rate; must satisfy \code{gamma > 1}.
#' @param q Moment order used in Ibragimov-Rio covariance bound; must satisfy
#'   \code{q > 2} and \code{gamma * (1 - 2/q) > 1}.
#' @param n Sample size used for the two-sided envelope.
#' @param C_alpha Strong-mixing constant (default 1).
#'
#' @return A list with components \code{K_TAC}, \code{K_MPC}, \code{c_L},
#'   \code{c_U}, \code{b_optimal}, \code{m_optimal}, and \code{regime}
#'   (a short label describing which method is favored).
#'
#' @examples
#' htdv_equivalence_constants(gamma = 2, q = 6, n = 60)
#'
#' @export
htdv_equivalence_constants <- function(gamma, q = 6, n = 100, C_alpha = 1) {
  .assert_positive_scalar(gamma, "gamma")
  .assert_positive_scalar(q, "q")
  .assert_positive_scalar(n, "n")
  .assert_positive_scalar(C_alpha, "C_alpha")
  if (gamma <= 1) {
    stop("'gamma' must be > 1 for summable mixing.", call. = FALSE)
  }
  if (q <= 2) {
    stop("'q' must be > 2.", call. = FALSE)
  }
  s <- gamma * (1 - 2 / q)
  if (s <= 1) {
    stop("Condition gamma*(1 - 2/q) > 1 violated; increase gamma or q.",
         call. = FALSE)
  }
  K_TAC <- 1 + 16 * C_alpha^(1 - 2 / q) * .zeta_riemann(s)
  K_MPC <- (gamma / (gamma - 1))^2 * (1 + 4 / (gamma - 1))
  c_U <- sqrt(max(K_TAC, K_MPC))
  penalty <- 8 * C_alpha^(1 - 2 / q) / (gamma - 1) *
    n^(-(gamma - 1) / (2 * gamma + 1))
  c_L <- sqrt(max(0, 1 - penalty))
  b_opt <- max(1, round(n^(1 / (2 * gamma + 1))))
  m_opt <- max(1, floor(n / b_opt))
  regime <- if (c_L < 0.5) {
    "pre-asymptotic: intervals must be widened"
  } else if (K_TAC < K_MPC) {
    "TAC favored"
  } else {
    "MPC favored"
  }
  list(K_TAC = K_TAC, K_MPC = K_MPC,
       c_L = c_L, c_U = c_U,
       b_optimal = b_opt, m_optimal = m_opt,
       gamma = gamma, q = q, n = n,
       regime = regime)
}

.zeta_riemann <- function(s) {
  if (s <= 1) return(Inf)
  sum(seq_len(10000L)^(-s)) +
    stats::integrate(function(x) x^(-s), lower = 10000, upper = Inf)$value
}

test_that("htdv_equivalence_constants returns positive K_TAC and K_MPC", {
  out <- htdv_equivalence_constants(gamma = 2, q = 6, n = 100)
  expect_true(out$K_TAC > 0)
  expect_true(out$K_MPC > 0)
  expect_true(out$c_U > 0)
})

test_that("higher gamma reduces K_TAC", {
  a <- htdv_equivalence_constants(gamma = 2, q = 6, n = 100)
  b <- htdv_equivalence_constants(gamma = 4, q = 6, n = 100)
  expect_lt(b$K_TAC, a$K_TAC)
})

test_that("larger n tightens c_L", {
  small <- htdv_equivalence_constants(gamma = 4, q = 6, n = 200)
  big   <- htdv_equivalence_constants(gamma = 4, q = 6, n = 50000)
  expect_gt(big$c_L, small$c_L)
  expect_true(big$c_L > 0)
})

test_that("htdv_equivalence_constants validates inputs", {
  expect_error(htdv_equivalence_constants(gamma = 0.5))
  expect_error(htdv_equivalence_constants(gamma = 2, q = 1))
  expect_error(htdv_equivalence_constants(gamma = 1.1, q = 2.01))
})

---
title: "HTDV — Empirical and Simulation Validation"
author: "José Mauricio Gómez Julián"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{HTDV — Empirical and Simulation Validation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE,
  purl = FALSE
)
```

# Purpose

`HTDV` is a hypothesis-testing framework for dependent and unbalanced data
under strong-mixing conditions. Three inferential layers run in parallel:
hierarchical Bayesian inference (HMC via Stan), HAR-Wald frequentist
inference (Andrews-Kiefer-Vogelsang), and stationary block bootstrap
(Patton-Politis-White). The framework exists in order to *triangulate* an
inferential answer when no single layer is universally valid in the
finite-sample regime.

This vignette summarizes the empirical evidence that the framework's
calibration claims are not stylistic but operationally testable. It
collects two pieces of validation:

1. A pre-registered factorial Monte Carlo study covering 1024 cells of
   the (sample size × dependence × tail × imbalance × shift) design.
2. Three external benchmarks against published literature on public
   data: post-1984 US CPI inflation against Stock & Watson (2007), long-
   run log-CAPE against Campbell & Shiller (1998), and the US-Canada
   10-year yield differential against the iid Welch baseline.

Both validations are reproducible end-to-end from scripts shipped in the
companion paper repository. The summary tables are loaded into the
package as datasets so users can interrogate them without re-running.

```{r packages, eval = TRUE}
library(HTDV)
data(htdv_sim_summary)
data(htdv_empirical_benchmarks)
```

# 1. Factorial simulation study

The full factorial design of Section 12-bis of the companion paper crosses
five factors at four levels each:

- **Sample size** $n \in \{40, 80, 200, 500\}$
- **AR(1) coefficient** $\phi \in \{0, 0.3, 0.6, 0.85\}$
- **Innovation tail** $\eta_t \sim \{\mathcal{N}(0,1),\,t_5,\,t_3,\,t_{2.1}\}$
- **Imbalance ratio** $n_1/n_2 \in \{1, 1.5, 3, 6\}$
- **Location shift** $\Delta \in \{0, 0.1, 0.25, 0.5\} \cdot \sigma_\infty$

For a total of $4^5 = 1024$ cells, each replicated $R = 500$ times against
three inferential layers (HAR, bootstrap, Bayes). The full run took
31.1 hours on a 16-core Linux workstation.

The summary table `htdv_sim_summary` carries one row per
(cell × layer) combination, with the following columns:

```{r summary-cols, eval = TRUE}
str(htdv_sim_summary)
```

## 1.1. Empirical size: the calibration story

```{r size-table, eval = TRUE}
size_cells <- htdv_sim_summary[htdv_sim_summary$delta == 0, ]
agg_phi <- aggregate(reject_rate ~ n + phi + layer, data = size_cells,
                     FUN = mean)
agg_phi$reject_rate <- round(agg_phi$reject_rate, 3)
reshape(agg_phi, idvar = c("n", "phi"), timevar = "layer",
        direction = "wide")
```

The Bayesian envelope holds nominal size (~0.05) across the entire grid.
HAR and bootstrap inflate dramatically under high persistence: at
$\phi=0.85$ and $n=40$, HAR rejects 60% and bootstrap rejects 47% of the
time when nominal is 5%.

## 1.2. Coverage (sign-corrected)

```{r coverage-by-layer, eval = TRUE}
agg_cov <- aggregate(coverage ~ layer, data = htdv_sim_summary,
                     FUN = function(x) c(mean = round(mean(x), 4),
                                          min = round(min(x), 3),
                                          max = round(max(x), 3)))
agg_cov
```

Bayesian coverage averages 94.4% (nominal 95%, observed minimum 88.4%).
HAR averages 82.0%; bootstrap 82.4%. Both classical layers fall short of
nominal under finite-sample dependence and—in the worst cells—lose
roughly a third of their nominal coverage.

## 1.3. HMC diagnostics

```{r diag-by-region, eval = TRUE}
b <- htdv_sim_summary[htdv_sim_summary$layer == "bayes", ]
agg_diag <- aggregate(diag_pass_rate ~ n + phi, data = b, FUN = mean)
agg_diag$diag_pass_rate <- round(agg_diag$diag_pass_rate, 3)
reshape(agg_diag, idvar = "n", timevar = "phi", direction = "wide")
```

Diagnostic-pass rate is below 0.7 only in the corner where
$\phi=0.85$ and $n \le 80$, the limit-of-identification zone for the
AR(1) likelihood. Use `htdv_simstudy_warnings()` on a fresh simulation
output to flag those cells automatically.

## 1.4. Pre-registered benchmark verdicts

| Benchmark | Pass rate | Reading |
|-----------|-----------|---------|
| **B1** size control across layers | 0.32 / 255 cells | Failed *as a conjunction*; passes for Bayes alone in 98% of cells |
| **B2** power monotonicity in $\Delta$ | 0.99 / 768 cell-layers | Effectively passes (residual is MC noise) |
| **B3** Bayes coverage $\ge 0.93$ | 0.89 / 1023 cells | Passes; minimum 0.884 in the hardest corner |

The conjunctive failure of B1 is the operational evidence that motivates
the framework: HAR and bootstrap, calibrated against asymptotic
critical values, are not safe inferential anchors under strong
dependence at finite sample size. The Bayesian envelope is.

## 1.5. Reproducing the simulation

The simulation is reproduced by the `run_simstudy.R` script of the
companion paper repository:

```{r repro-sim, eval = FALSE}
res <- htdv_simstudy(
  n_grid    = c(40, 80, 200, 500),
  phi_grid  = c(0, 0.3, 0.6, 0.85),
  tail_grid = c("normal", "t5", "t3", "t2_1"),
  imb_grid  = c(1, 1.5, 3, 6),
  delta_grid = c(0, 0.1, 0.25, 0.5),
  R         = 500,
  seed      = 20260422,
  out_dir   = "simstudy_cells_dir"
)
summ <- htdv_simstudy_summary(res)
```

The per-cell results are written atomically to one RDS per cell in
`out_dir`; partial runs are fully resumable.

# 2. Empirical benchmarks

Three datasets, each compared against a pre-existing reference value
from the published literature on the same vintage. The summary table is
shipped as `htdv_empirical_benchmarks`:

```{r empirical-table, eval = TRUE}
htdv_empirical_benchmarks[, c("dataset", "n", "reference",
                              "reference_value", "har_ci_lo", "har_ci_hi",
                              "bayes_ci_lo", "bayes_ci_hi",
                              "boot_ci_lo", "boot_ci_hi",
                              "agreement_har", "agreement_bayes",
                              "agreement_boot")]
```

All three layers reproduce all three references with `agreement` in every
case. The substantively interesting finding is the *width* of the
agreement, which scales with the persistence of the underlying series.

## 2.1. The persistence ladder

| Dataset | $\widehat\phi$ | HAR half-width | Bayes half-width | Bayes/HAR |
|---------|----------------|----------------|------------------|------------------|
| FRED-MD CPI inflation, post-1984 | $\approx 0.45$ | 0.57 | 0.46 | 0.81 |
| Shiller log-CAPE, 1881-latest | $\approx 0.97$ | 0.25 | 0.70 | 2.80 |
| US-CA 10y yield differential | $\approx 0.99$ | 0.49 | 7.35 | 15.0 |

At moderate persistence, HAR and Bayes agree to within 20%. At
near-unit-root, the Bayesian envelope is 15× wider than HAR. Both layers
are technically asymptotically valid; only Bayes accounts honestly for
the finite-sample uncertainty inflation that occurs when $\phi \to 1$.

## 2.2. Practical guidance

If you are running HTDV on a new dataset, the following workflow gives
the most reliable inference:

1. Run the three layers (`htdv_lrv()` + Wald construction; `htdv_boot()`;
   `htdv_fit()`).
2. Compare the widths of the three intervals. Disagreement at the
   one-third-of-an-order-of-magnitude scale is a real signal: the
   underlying series is in a regime where HAR's asymptotic critical
   values are losing reliability.
3. In that regime, prefer the Bayesian envelope as the primary
   inferential answer and use HAR or bootstrap as cross-checks rather
   than the other way around.

This is exactly what the framework is designed for: making the
calibration disagreement *visible* at the moment of inference.

# 3. Reproducing the empirical benchmarks

Required public files (downloaded from public sources, named per the
script `run_benchmarks.R`):

| Data file | Source URL | Provenance |
|-----------|------------|------------|
| `2026-03-MD.csv` | `stlouisfed.org/research/economists/mccracken/fred-databases` | McCracken-Ng FRED-MD vintage 2026-03 |
| `ie_data.xls` | `shillerdata.com` | Robert Shiller's online dataset |
| `us_canada_10y.csv` | `fred.stlouisfed.org/graph/fredgraph.csv?id=GS10,IRLTLT01CAM156N` | FRED multi-series download |

Total wall-clock for the three benchmarks combined is approximately
10-20 minutes on a single core; the heavy step is the two-sample HMC fit
for the US-CA yield comparison.

# References

McCracken, M. W., & Ng, S. (2016). FRED-MD: A monthly database for
macroeconomic research. *Journal of Business & Economic Statistics*,
34(4), 574-589.

Stock, J. H., & Watson, M. W. (2007). Why has US inflation become harder
to forecast? *Journal of Money, Credit and Banking*, 39, 3-33.

Campbell, J. Y., & Shiller, R. J. (1998). Valuation ratios and the
long-run stock market outlook. *Journal of Portfolio Management*,
24(2), 11-26.

---
title: "Introduction to HTDV"
author: "José Mauricio Gómez Julián"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{Introduction to HTDV}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE,
  purl = FALSE
)
```

# Overview

`HTDV` provides hypothesis testing and estimation for dependent, unbalanced
data through four integrated layers: (i) spectrally-faithful likelihoods
(Whittle, composite), (ii) hierarchical Bayesian priors on the dependence
nuisance parameters, (iii) Hamiltonian Monte Carlo with mandatory diagnostic
gates, and (iv) a Bayesian decision layer (ROPE, bridge-sampling Bayes
factors, WAIC, leave-future-out cross-validation, stacking). Finite-sample
robustness is delivered by fixed-bandwidth HAR inference, block bootstrap
with automatic block length, and adaptive conformal prediction.

# Quick start: long-run variance and fixed-b HAR

```{r}
library(HTDV)

x <- stats::arima.sim(model = list(ar = 0.6), n = 200,
                      rand.gen = stats::rnorm)
x <- as.numeric(x)

lrv <- htdv_lrv(x, kernel = "qs", bandwidth = "andrews")
lrv$lrv

test <- htdv_fixedb(x, theta0 = 0, B = 0.1,
                    sims = 1000L, seed = 1L)
test$p_value
```

# Block bootstrap

```{r}
boot <- htdv_boot(x, mean, R = 1999L,
                  type = "stationary",
                  block_length = "auto",
                  seed = 1L)
boot$ci_percentile
boot$block_length
```

# Adaptive conformal

```{r}
pred <- function(history) {
  if (length(history) >= 1L) history[length(history)] else 0
}
conf <- htdv_conformal(x, pred, alpha_target = 0.1,
                       lambda = 0.05, burn_in = 20L)
conf$coverage
```

# Hierarchical Bayesian fit

The default `htdv_fit()` compiles a Stan model on first use and caches it for
subsequent calls.

```{r}
fit <- htdv_fit(x, model = "tac", chains = 4L, iter = 4000L,
                refresh = 0L, seed = 1L)
htdv_summary(fit, rope = c(-0.1, 0.1), target = "theta")
htdv_diagnostics(fit)$passed
htdv_ppc(fit, x)
```

# Envelope across likelihoods

```{r}
fit_w <- htdv_fit(x, model = "whittle", chains = 4L, iter = 4000L,
                  refresh = 0L, seed = 2L)
fit_c <- htdv_fit(x, model = "composite", chains = 4L, iter = 4000L,
                  refresh = 0L, seed = 3L)

env <- htdv_envelope(list(whittle = fit_w, composite = fit_c),
                     target = "theta")
env$intervals
```

# Decision: ROPE, Bayes factor, stacking

```{r}
draws <- as.numeric(rstan::extract(fit$stanfit, pars = "theta")$theta)
htdv_rope(draws, rope = c(-0.1, 0.1))$decision

if (requireNamespace("bridgesampling", quietly = TRUE)) {
  htdv_bf(fit_w, fit_c)$bf10
}

if (requireNamespace("loo", quietly = TRUE)) {
  htdv_stack(list(whittle = fit_w, composite = fit_c))$weights
}
```

# Explicit equivalence constants

```{r}
htdv_equivalence_constants(gamma = 2, q = 6, n = length(x))
```

# References

Andrews, D. W. K. (1991). Econometrica 59(3): 817-858.

Berger, J. O. (1994). Test 3(1): 5-124.

Gibbs, I., & Candes, E. (2021). NeurIPS 34: 1660-1672.

Kiefer, N. M., & Vogelsang, T. J. (2005). Econometric Theory 21(6): 1130-1164.

Kruschke, J. K. (2018). Advances in Methods and Practices in Psychological
Science 1(2): 270-280.

Patton, A., Politis, D. N., & White, H. (2009). Econometric Reviews 28(4):
372-375.

Varin, C., Reid, N., & Firth, D. (2011). Statistica Sinica 21(1): 5-42.

Vehtari, A., Gelman, A., & Gabry, J. (2017). Statistics and Computing 27(5):
1413-1432.

Watanabe, S. (2010). JMLR 11: 3571-3594.

Whittle, P. (1953). Arkiv foer Matematik 2(5): 423-434.

Yao, Y., Vehtari, A., Simpson, D., & Gelman, A. (2018). Bayesian Analysis
13(3): 917-1007.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/equivalence_constants.R
\name{htdv_equivalence_constants}
\alias{htdv_equivalence_constants}
\title{Numerical Constants for TAC/WSC/MPC Metric Equivalence}
\usage{
htdv_equivalence_constants(gamma, q = 6, n = 100, C_alpha = 1)
}
\arguments{
\item{gamma}{Mixing decay rate; must satisfy \code{gamma > 1}.}

\item{q}{Moment order used in Ibragimov-Rio covariance bound; must satisfy
\code{q > 2} and \code{gamma * (1 - 2/q) > 1}.}

\item{n}{Sample size used for the two-sided envelope.}

\item{C_alpha}{Strong-mixing constant (default 1).}
}
\value{
A list with components \code{K_TAC}, \code{K_MPC}, \code{c_L},
  \code{c_U}, \code{b_optimal}, \code{m_optimal}, and \code{regime}
  (a short label describing which method is favored).
}
\description{
Computes the explicit finite-sample constants K_TAC(gamma, q), K_MPC(gamma)
and the unified two-sided envelope (c_L, c_U) from Theorems 5-8 of the
companion paper.
}
\examples{
htdv_equivalence_constants(gamma = 2, q = 6, n = 60)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/block_bootstrap.R
\name{htdv_boot}
\alias{htdv_boot}
\title{Block Bootstrap with Automatic Block Length}
\usage{
htdv_boot(
  x,
  statistic,
  R = 1999L,
  type = c("circular", "stationary"),
  block_length = "auto",
  level = 0.95
)
}
\arguments{
\item{x}{Numeric vector.}

\item{statistic}{Function \code{function(x)} returning a scalar or numeric
vector.}

\item{R}{Number of bootstrap replicates.}

\item{type}{\code{"circular"} or \code{"stationary"}.}

\item{block_length}{Either \code{"auto"} (Patton-Politis-White) or a
positive integer.}

\item{level}{Confidence level for intervals.}
}
\value{
A list with \code{t0} (statistic on original data), \code{t_star}
  (bootstrap distribution), \code{ci_percentile}, \code{ci_basic},
  \code{ci_studentized}, and \code{block_length}.
}
\description{
Implements circular and stationary block bootstraps (Politis-Romano 1992,
1994) with Patton-Politis-White (2009) automatic block length selection.
}
\examples{
set.seed(1)
x <- arima.sim(model = list(ar = 0.5), n = 200, rand.gen = rnorm)
out <- htdv_boot(as.numeric(x), mean, R = 500, type = "stationary")
out$ci_percentile

}
\references{
Politis, D.N., & Romano, J.P. (1994). The Stationary Bootstrap. JASA
89(428): 1303-1313.
Patton, A., Politis, D.N., & White, H. (2009). Correction to Automatic
Block-Length Selection for the Dependent Bootstrap. Econometric Reviews
28(4): 372-375.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{htdv_sim_summary}
\alias{htdv_sim_summary}
\title{Aggregated Output of the HTDV Factorial Simulation Study}
\format{
A data frame with one row per (cell, layer) and the following
columns:
\describe{
  \item{n}{Per-group primary sample size (40, 80, 200, 500).}
  \item{phi}{AR(1) coefficient (0, 0.3, 0.6, 0.85).}
  \item{tail}{Innovation distribution: \code{"normal"}, \code{"t5"},
    \code{"t3"}, \code{"t2_1"}.}
  \item{imb}{Imbalance ratio n1/n2 (1, 1.5, 3, 6).}
  \item{delta}{Location shift in units of sigma_inf (0, 0.1, 0.25, 0.5).}
  \item{layer}{Inferential layer: \code{"har"}, \code{"boot"},
    \code{"bayes"}.}
  \item{n_reps}{Number of replications aggregated (500 for all cells).}
  \item{reject_rate}{Empirical rejection rate at nominal alpha = 0.05.}
  \item{coverage}{Fraction of replications whose 95\% interval covered
    the true parameter (sign-corrected for HAR and bootstrap).}
  \item{mean_ci_length}{Average length of the 95\% interval.}
  \item{mean_runtime}{Mean per-replication runtime in seconds.}
  \item{diag_pass_rate}{Fraction of replications passing all HMC
    diagnostic gates (Bayesian layer only; \code{NA} otherwise).}
}
}
\source{
Generated by \code{run_simstudy.R} of the companion paper
  repository on master seed \code{20260422}, with sign-corrected
  coverage produced by \code{recover_coverage.R}. Total wall-clock
  31.1 hours on a 16-core Linux workstation. See
  \code{vignette("HTDV-validation")} for the full report.
}
\usage{
htdv_sim_summary
}
\description{
Sign-corrected aggregated results of the factorial Monte Carlo study
described in Section 12-bis of the companion paper. The study crosses
five factors at four levels each (sample size, AR(1) coefficient,
innovation tail, imbalance ratio, location shift) for 1024 cells, each
replicated 500 times against three inferential layers (HAR, bootstrap,
hierarchical Bayesian). One cell (n=500, phi=0.85, t3, imb=1, delta=0)
was lost to an I/O collision and is excluded; the table contains
1023 cells x 3 layers = 3069 rows.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{htdv_ppc}
\alias{htdv_ppc}
\title{Posterior-Predictive Checks on Dependence Statistics}
\usage{
htdv_ppc(fit, x_obs, lag_max = 10L)
}
\arguments{
\item{fit}{An \code{htdv_fit} object whose posterior-predictive replicates
are stored as parameter \code{x_rep}.}

\item{x_obs}{The observed data vector.}

\item{lag_max}{Maximum lag for autocorrelation.}
}
\value{
A list with \code{p_acf}, \code{p_ljung_box}, \code{p_spectral},
  \code{p_hurst}.
}
\description{
Replicates autocorrelation, Ljung-Box, spectral discrepancy, and a
detrended-fluctuation Hurst exponent under the posterior and computes
posterior-predictive \code{p}-values.
}
\examples{
\donttest{
x <- rnorm(50)
fit <- htdv_fit(x, model = "tac", chains = 2, iter = 500,
                refresh = 0, seed = 1)
htdv_ppc(fit, x)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simstudy.R
\name{htdv_simstudy}
\alias{htdv_simstudy}
\title{Factorial Monte Carlo Simulation Study for Dependent-Unbalanced Data}
\usage{
htdv_simstudy(
  n_grid = c(40L, 80L, 200L, 500L),
  phi_grid = c(0, 0.3, 0.6, 0.85),
  tail_grid = c("normal", "t5", "t3", "t2_1"),
  imb_grid = c(1, 1.5, 3, 6),
  delta_grid = c(0, 0.1, 0.25, 0.5),
  R = 500L,
  seed = 20260422L,
  n_cores = 1L,
  layers = c("har", "boot", "bayes"),
  bayes_chains = 2L,
  bayes_iter = 600L,
  bayes_warmup = 300L,
  boot_R = 999L,
  rope = c(-0.1, 0.1),
  alpha = 0.05,
  out_dir = NULL,
  progress = TRUE
)
}
\arguments{
\item{n_grid}{Integer vector of primary sample sizes \eqn{n_1}.}

\item{phi_grid}{Numeric vector of AR(1) coefficients in \eqn{(-1,1)}.}

\item{tail_grid}{Character vector; any subset of \code{"normal"},
\code{"t5"}, \code{"t3"}, \code{"t2_1"}.}

\item{imb_grid}{Numeric vector of imbalance ratios \eqn{n_1/n_2}.}

\item{delta_grid}{Numeric vector of location shifts in units of
\eqn{\sigma_\infty}.}

\item{R}{Integer number of Monte Carlo replications per cell.}

\item{seed}{Integer master seed.}

\item{n_cores}{Integer number of workers for cell-level parallelism.
On non-Unix platforms parallelism falls back to serial.}

\item{layers}{Character vector; any subset of \code{c("har","boot","bayes")}.}

\item{bayes_chains}{Number of HMC chains per Bayesian fit.}

\item{bayes_iter}{Total HMC iterations (warmup + sampling).}

\item{bayes_warmup}{HMC warmup iterations.}

\item{boot_R}{Bootstrap replicates per call.}

\item{rope}{Length-2 numeric vector; ROPE for the Bayesian decision on
the raw (non-standardized) delta.}

\item{alpha}{Significance level (nominal size of tests; \eqn{1-\alpha} is
the coverage target for intervals).}

\item{out_dir}{Optional directory for per-cell incremental RDS results.
Each cell is written to its own \code{cell_<id>.rds} so that concurrent
workers cannot collide on a shared file. Final aggregation reads the
directory back, so successful cells are preserved even if some workers
later die. If \code{NULL}, no per-cell artifacts are written and the
only result is the in-memory return value.}

\item{progress}{Logical; print a one-line status per completed cell.}
}
\value{
A data frame with one row per (cell, replication, layer). Columns:
\code{cell_id, n, phi, tail, imb, n1, n2, delta, sigma_inf, layer,
replicate, reject, ci_lo, ci_hi, estimate, covered, ci_length,
rhat_max, ess_min, divergences, diag_pass, runtime_sec}.
}
\description{
Runs the pre-registered factorial Monte Carlo study of Section 12-bis of
the companion paper. The study crosses four factors (per-group sample
size, AR(1) coefficient, innovation tail, imbalance ratio) and evaluates
three inferential layers (HAR-Wald, stationary block bootstrap,
hierarchical Bayesian HMC) on each replication.
}
\section{Data-generating process}{

Two independent AR(1) series \eqn{x^{(1)}_t} of length \eqn{n_1=n} and
\eqn{x^{(2)}_t} of length \eqn{n_2 = \max(2, \mathrm{round}(n / \mathrm{imb}))}
are generated. The innovations are drawn from \code{rnorm},
scaled Student-\eqn{t_{(5)}}, scaled Student-\eqn{t_{(3)}} or scaled
Student-\eqn{t_{(2.1)}} so that the nominal innovation variance is one
when the law has a finite second moment. Group one has mean zero; group
two has mean \eqn{\Delta\cdot\sigma_\infty} where
\eqn{\sigma_\infty = 1/\sqrt{1 - \phi^2}}.
}

\section{Inferential layers}{

\itemize{
  \item \strong{HAR-Wald (\code{"har"}).} Welch-style two-sample statistic
    \eqn{z = (\bar x_1 - \bar x_2) / \sqrt{\hat\sigma_1^2/n_1 + \hat\sigma_2^2/n_2}}
    with long-run variances estimated via \code{\link{htdv_lrv}} (QS
    kernel, Andrews bandwidth) on each group's demeaned series.
    Rejection at asymptotic \eqn{\chi^2_1} critical value.
  \item \strong{Stationary block bootstrap (\code{"boot"}).} Each group
    resampled independently using
    \code{\link{htdv_boot}} with \code{block_length = "auto"} (Patton-
    Politis-White). Percentile interval on
    \eqn{\bar x_1^* - \bar x_2^*}.
  \item \strong{Hierarchical Bayesian (\code{"bayes"}).} Two-sample AR(1)
    Stan model (\code{simstudy_two_sample.stan}); posterior interval on
    \eqn{\delta = \alpha_2 - \alpha_1}. A run is flagged
    \code{diag_pass = TRUE} only if \eqn{\hat R < 1.01}, bulk and tail
    ESS above \eqn{200}, and zero divergences.
}
}

\examples{
\donttest{
res <- htdv_simstudy(n_grid = c(40, 80),
                     phi_grid = c(0, 0.6),
                     tail_grid = c("normal"),
                     imb_grid = c(1, 3),
                     delta_grid = c(0, 0.25),
                     R = 5L, n_cores = 1L,
                     layers = c("har", "boot"))
head(htdv_simstudy_summary(res))
}

}
\references{
Kiefer, N.M., & Vogelsang, T.J. (2005). Econometric Theory 21(6):
1130-1164.
Patton, A., Politis, D.N., & White, H. (2009). Econometric Reviews
28(4): 372-375.
Kruschke, J.K. (2018). Advances in Methods and Practices in
Psychological Science 1(2): 270-280.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.R
\name{htdv_envelope}
\alias{htdv_envelope}
\title{Berger-Robust Envelope Posterior}
\usage{
htdv_envelope(fits, target = "theta")
}
\arguments{
\item{fits}{List of \code{htdv_fit} objects on the same data.}

\item{target}{Name of the Stan parameter over which to build the envelope.}
}
\value{
A list with \code{draws} (envelope draws for the target parameter),
  \code{weights}, \code{intervals}, and \code{component_intervals}.
}
\description{
Builds an envelope posterior over a list of \code{htdv_fit} objects by
entropy-maximizing mixture weights subject to predictive consistency
(Berger 1994).
}
\examples{
\donttest{
x <- rnorm(50)
f1 <- htdv_fit(x, model = "tac", chains = 2, iter = 500,
               refresh = 0, seed = 1)
f2 <- htdv_fit(x, model = "wsc", chains = 2, iter = 500,
               refresh = 0, seed = 2)
htdv_envelope(list(f1, f2), target = "theta")$intervals
}

}
\references{
Berger, J.O. (1994). An Overview of Robust Bayesian Analysis. Test 3(1):
5-124.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decision.R
\name{htdv_bf}
\alias{htdv_bf}
\title{Bridge-Sampling Bayes Factor}
\usage{
htdv_bf(fit1, fit0, ...)
}
\arguments{
\item{fit1}{Model in the numerator.}

\item{fit0}{Model in the denominator.}

\item{...}{Additional arguments passed to
\code{bridgesampling::bridge_sampler}.}
}
\value{
A list with \code{bf10}, \code{log_bf10}, \code{logml1},
  \code{logml0}, and \code{error_percentage}.
}
\description{
Computes the Bayes factor between two fitted \code{\link{htdv_fit}}
objects via bridge sampling. Requires the \code{bridgesampling} package.
}
\examples{
\donttest{
if (requireNamespace("bridgesampling", quietly = TRUE)) {
  x <- rnorm(50)
  f1 <- htdv_fit(x, model = "tac", chains = 2, iter = 500,
                 refresh = 0, seed = 1)
  f0 <- htdv_fit(x, model = "tac", chains = 2, iter = 500,
                 refresh = 0, seed = 2)
  htdv_bf(f1, f0)$bf10
}
}

}
\references{
Meng, X.-L., & Wong, W.H. (1996). Simulating Ratios of Normalizing
Constants. Statistica Sinica 6(4): 831-860.
Gronau, Q.F. et al. (2017). A Tutorial on Bridge Sampling. JMP 81: 80-97.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{htdv_empirical_benchmarks}
\alias{htdv_empirical_benchmarks}
\title{HTDV Empirical Benchmarks Against Published References}
\format{
A data frame with three rows (one per dataset) and the
following columns:
\describe{
  \item{dataset}{Name of the benchmark dataset.}
  \item{reference}{Citation of the reference paper.}
  \item{n}{Sample size after preprocessing.}
  \item{sample}{Sample window (string).}
  \item{reference_value}{Reference point value.}
  \item{reference_se}{Reference standard error (\code{NA} when the
    reference does not provide one).}
  \item{har_point, har_ci_lo, har_ci_hi}{HAR-Wald point and 95\% CI.}
  \item{bayes_point, bayes_ci_lo, bayes_ci_hi}{Bayesian point and
    95\% credible interval.}
  \item{bayes_diag_pass, bayes_rhat_max, bayes_ess_min,
    bayes_divergences}{Bayesian HMC diagnostics.}
  \item{boot_point, boot_ci_lo, boot_ci_hi}{Stationary block bootstrap
    point and 95\% percentile CI.}
  \item{agreement_har, agreement_bayes, agreement_boot}{Per-layer
    agreement verdict: \code{"agreement"}, \code{"htdv-strict (above ref)"},
    or \code{"htdv-strict (below ref)"}.}
}
}
\source{
Generated by \code{run_benchmarks.R} of the companion paper
  repository. See \code{vignette("HTDV-validation")} for the
  methodological narrative and the persistence-ladder reading.
}
\usage{
htdv_empirical_benchmarks
}
\description{
Output of the three-dataset external validation protocol of
Section 12-ter of the companion paper. Each dataset is run against a
pre-existing reference result from the published literature on the
same vintage, with the three HTDV inferential layers (HAR, bootstrap,
hierarchical Bayesian) reporting their point estimate, 95\% interval,
and a Bayesian diagnostic-pass flag.
}
\details{
The three benchmarks (E1, E2, E3) and their references are:
\itemize{
  \item{\strong{E1.} Post-1984 US CPI inflation against Stock and
    Watson (2007). FRED-MD vintage 2026-03, variable \code{CPIAUCSL}
    transformed to annualized monthly inflation.}
  \item{\strong{E2.} Long-run mean of log-CAPE against Campbell and
    Shiller (1998). Robert Shiller's online historical dataset.}
  \item{\strong{E3.} US vs. Canada 10-year government bond yield
    differential against the iid Welch baseline. FRED series
    \code{GS10} and \code{IRLTLT01CAM156N}, common sample 1990-01 to
    latest.}
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HTDV-package.R
\docType{package}
\name{HTDV-package}
\alias{HTDV-package}
\alias{HTDV}
\title{HTDV: Hypothesis Testing for Dependent Variables with Unbalanced Data}
\description{
Implements hierarchical Bayesian inference, robust frequentist inference,
and distribution-free inference for dependent and unbalanced data under
strong-mixing conditions.
}
\section{Main functions}{

\itemize{
  \item \code{\link{htdv_fit}}: unified hierarchical Bayesian fit via HMC.
  \item \code{\link{htdv_tac}}, \code{\link{htdv_wsc}}, \code{\link{htdv_mpc}}:
        convergence-regime-specific fits.
  \item \code{\link{htdv_whittle}}, \code{\link{htdv_composite}}:
        likelihood surrogates for dependent data.
  \item \code{\link{htdv_envelope}}: Berger-robust posterior envelope.
  \item \code{\link{htdv_lrv}}: long-run variance with Newey-West and
        Andrews bandwidth.
  \item \code{\link{htdv_fixedb}}: fixed-bandwidth HAR inference
        (Kiefer-Vogelsang 2005).
  \item \code{\link{htdv_boot}}: block bootstrap with Patton-Politis-White
        automatic block length.
  \item \code{\link{htdv_conformal}}: adaptive conformal inference
        (Gibbs-Candes 2021).
  \item \code{\link{htdv_rope}}: posterior decision under Region of
        Practical Equivalence (Kruschke 2018).
  \item \code{\link{htdv_bf}}: bridge-sampling Bayes factor.
  \item \code{\link{htdv_waic_lfo}}: WAIC and leave-future-out
        cross-validation.
  \item \code{\link{htdv_stack}}: predictive stacking across models.
  \item \code{\link{htdv_diagnostics}}, \code{\link{htdv_ppc}}:
        MCMC diagnostics and posterior-predictive checks.
  \item \code{\link{htdv_equivalence_constants}}: explicit numerical
        constants for TAC/WSC/MPC metric equivalence.
}
}

\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/IsadoreNabi/HTDV}
  \item Report bugs at \url{https://github.com/IsadoreNabi/HTDV/issues}
}

}
\author{
\strong{Maintainer}: Jose Mauricio Gomez Julian \email{isadorenabi@pm.me} (\href{https://orcid.org/0009-0000-2412-3150}{ORCID})

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simstudy.R
\name{htdv_simstudy_warnings}
\alias{htdv_simstudy_warnings}
\title{Flag Simulation-Study Cells in the Limit-of-Identification Zone}
\usage{
htdv_simstudy_warnings(summ, threshold = 0.7)
}
\arguments{
\item{summ}{Data frame from \code{\link{htdv_simstudy_summary}}.}

\item{threshold}{Minimum acceptable Bayesian \code{diag_pass_rate}.
Default \code{0.7}.}
}
\value{
A data frame with the columns \code{n, phi, tail, imb, delta,
  layer, diag_pass_rate}, restricted to Bayesian rows whose pass rate is
  below \code{threshold}, plus an integer column \code{flag_index}
  (sequential identifier of flagged cells, useful for cross-referencing
  in reports and in vignettes).
}
\description{
Reads an aggregated \code{\link{htdv_simstudy_summary}} output and returns
the subset of cells where the Bayesian diagnostic-pass rate falls below a
user-set threshold. Such cells are typically located in the corner of the
design where strong autocorrelation meets a small sample (high \eqn{\phi}
and low \eqn{n}); the AR(1) likelihood there approaches its limit of
identification, MCMC diagnostics flag the resulting posterior, and
practitioners should treat the cell with extra caution—either by
increasing HMC iterations, tightening the priors on \eqn{\phi}, or
falling back to the conformal anchor whose validity does not require
identifiability.
}
\examples{
\donttest{
res <- htdv_simstudy(n_grid = c(40, 200),
                     phi_grid = c(0, 0.85),
                     tail_grid = "normal",
                     imb_grid = c(1, 6),
                     delta_grid = c(0, 0.25),
                     R = 50L)
summ <- htdv_simstudy_summary(res)
htdv_simstudy_warnings(summ, threshold = 0.7)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decision.R
\name{htdv_waic_lfo}
\alias{htdv_waic_lfo}
\title{WAIC and Leave-Future-Out Cross-Validation}
\usage{
htdv_waic_lfo(fit, L = NULL, k_threshold = 0.7)
}
\arguments{
\item{fit}{An \code{htdv_fit} object with pointwise log-likelihood stored
as parameter \code{log_lik}.}

\item{L}{Block-prefix length; leave-future-out starts at time \code{L+1}.
Default \code{floor(n/5)}.}

\item{k_threshold}{Pareto-k refresh threshold; when exceeded, importance
sampling is refreshed.}
}
\value{
A list with \code{waic}, \code{elpd_lfo}, \code{elpd_se},
  \code{pareto_k}, and \code{refresh_times}.
}
\description{
Computes the Widely Applicable Information Criterion (Watanabe 2010) and
leave-future-out expected log predictive density (Buerkner-Gabry-Vehtari
2020) from a fitted \code{\link{htdv_fit}} object.
}
\examples{
\donttest{
if (requireNamespace("loo", quietly = TRUE)) {
  x <- rnorm(50)
  fit <- htdv_fit(x, model = "tac", chains = 2, iter = 500,
                  refresh = 0, seed = 1)
  htdv_waic_lfo(fit)$waic
}
}

}
\references{
Watanabe, S. (2010). Asymptotic Equivalence of Bayes Cross Validation and
Widely Applicable Information Criterion. JMLR 11: 3571-3594.
Buerkner, P.-C., Gabry, J., & Vehtari, A. (2020). Approximate
Leave-Future-Out Cross-Validation for Bayesian Time Series Models. Journal
of Statistical Computation and Simulation 90(14): 2499-2523.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/whittle.R
\name{htdv_whittle}
\alias{htdv_whittle}
\title{Whittle Likelihood for Stationary Series}
\usage{
htdv_whittle(x, spec_fun, theta, taper = FALSE)
}
\arguments{
\item{x}{Numeric vector.}

\item{spec_fun}{Function \code{function(lambda, theta)} returning spectral
density values at angular frequencies \code{lambda}.}

\item{theta}{Numeric parameter vector passed to \code{spec_fun}.}

\item{taper}{Logical; if \code{TRUE}, a 10-percent cosine taper is applied.}
}
\value{
A list with \code{loglik} (scalar), \code{frequencies},
  \code{periodogram}, and \code{fitted_spectrum}.
}
\description{
Computes the Whittle pseudo-log-likelihood at Fourier frequencies for a
user-supplied spectral density.
}
\examples{
x <- arima.sim(model = list(ar = 0.5), n = 200, rand.gen = rnorm)
ar1_spec <- function(lambda, theta) {
  phi <- theta[1]; sigma2 <- theta[2]
  sigma2 / (2 * pi * (1 - 2 * phi * cos(lambda) + phi^2))
}
htdv_whittle(as.numeric(x), ar1_spec, c(0.5, 1))$loglik

}
\references{
Whittle, P. (1953). Estimation and Information in Stationary Time Series.
Arkiv foer Matematik, 2(5), 423-434.
Dzhaparidze, K. (1986). Parameter Estimation and Hypothesis Testing in
Spectral Analysis of Stationary Time Series. Springer.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/composite.R
\name{htdv_composite}
\alias{htdv_composite}
\title{Composite Log-Likelihood for Dependent Data}
\usage{
htdv_composite(x, density_fun, theta, k = 1L, log = FALSE)
}
\arguments{
\item{x}{Numeric vector.}

\item{density_fun}{Function \code{function(block, theta)} returning the
joint density of a \code{(k+1)}-block of consecutive observations.}

\item{theta}{Numeric parameter vector.}

\item{k}{Window width (pairwise if \code{k = 1}).}

\item{log}{Logical; if \code{TRUE}, \code{density_fun} already returns log
densities.}
}
\value{
A list with \code{loglik} (scalar), \code{n_blocks}, and
  \code{block_values}.
}
\description{
Pairwise or \code{k}-tuple composite log-likelihood (Varin, Reid and Firth
2011) for strictly stationary sequences.
}
\examples{
x <- arima.sim(model = list(ar = 0.4), n = 200, rand.gen = rnorm)
gauss_ar1 <- function(block, theta) {
  phi <- theta[1]; sigma2 <- theta[2]
  a <- block[1]; b <- block[2]
  dnorm(a, 0, sqrt(sigma2 / (1 - phi^2)), log = TRUE) +
    dnorm(b, phi * a, sqrt(sigma2), log = TRUE)
}
htdv_composite(as.numeric(x), gauss_ar1, c(0.4, 1), k = 1, log = TRUE)$loglik

}
\references{
Varin, C., Reid, N., & Firth, D. (2011). An overview of composite
likelihood methods. Statistica Sinica, 21(1), 5-42.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.R
\name{htdv_fit}
\alias{htdv_fit}
\alias{htdv_tac}
\alias{htdv_wsc}
\alias{htdv_mpc}
\title{Hierarchical Bayesian Fit for Dependent Variables}
\usage{
htdv_fit(
  x,
  model = c("tac", "wsc", "mpc", "whittle", "composite"),
  chains = 4L,
  iter = 4000L,
  warmup = iter\%/\%2L,
  refresh = max(iter\%/\%10L, 1L),
  adapt_delta = 0.99,
  max_treedepth = 12L,
  seed = NULL,
  ...
)

htdv_tac(x, ...)

htdv_wsc(x, ...)

htdv_mpc(x, ...)
}
\arguments{
\item{x}{Numeric vector of observations.}

\item{model}{One of \code{"tac"}, \code{"wsc"}, \code{"mpc"},
\code{"whittle"}, \code{"composite"}.}

\item{chains}{Number of HMC chains.}

\item{iter}{Total iterations per chain (warmup plus sampling).}

\item{warmup}{Number of warmup iterations.}

\item{refresh}{Stan progress refresh rate (0 suppresses output).}

\item{adapt_delta}{HMC adaptation target acceptance.}

\item{max_treedepth}{Maximum NUTS treedepth.}

\item{seed}{Optional integer seed forwarded to \pkg{rstan}.}

\item{...}{Additional arguments forwarded to \code{rstan::sampling}.}
}
\value{
An object of class \code{htdv_fit}, a list with components
  \code{stanfit}, \code{model}, \code{data}, \code{n}, \code{call}.
}
\description{
Fits the hierarchical Bayesian model described in the companion paper via
Hamiltonian Monte Carlo (NUTS) using \pkg{rstan}. Nuisance parameters
\code{sigma_inf}, \code{gamma_mix}, and (for WSC) \code{b} carry priors and
are integrated over.
}
\examples{
\donttest{
x <- rnorm(50)
fit <- htdv_fit(x, model = "tac", chains = 2, iter = 500,
                refresh = 0, seed = 1)
class(fit)
}

}
\references{
Hoffman, M.D., & Gelman, A. (2014). The No-U-Turn Sampler. JMLR 15(1):
1593-1623.
Betancourt, M. (2016). Diagnosing Suboptimal Cotangent Disintegrations in
HMC. <doi:10.48550/arXiv.1604.00695>.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.R
\name{htdv_summary}
\alias{htdv_summary}
\title{Print-Friendly Summary for an HTDV Fit}
\usage{
htdv_summary(fit, rope = NULL, target = "theta")
}
\arguments{
\item{fit}{An \code{htdv_fit} object.}

\item{rope}{Optional two-element vector for ROPE decision on the target.}

\item{target}{Parameter name to summarize.}
}
\value{
A data frame with posterior mean, sd, 2.5\%, 50\%, 97.5\%, Rhat,
  bulk and tail ESS for \code{target}, plus the ROPE decision if requested.
}
\description{
Print-Friendly Summary for an HTDV Fit
}
\examples{
\donttest{
x <- rnorm(50)
fit <- htdv_fit(x, model = "tac", chains = 2, iter = 500,
                refresh = 0, seed = 1)
htdv_summary(fit, target = "theta")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decision.R
\name{htdv_rope}
\alias{htdv_rope}
\title{Region-of-Practical-Equivalence (ROPE) Decision}
\usage{
htdv_rope(draws, rope, level = 0.95)
}
\arguments{
\item{draws}{Numeric vector of posterior draws of the parameter of
interest.}

\item{rope}{Length-2 numeric vector \code{c(lower, upper)} defining the
practical-equivalence region.}

\item{level}{Credibility level for the highest-density interval.}
}
\value{
A list with \code{decision} (\code{"accept"}, \code{"reject"},
  \code{"undecided"}), \code{hdi}, \code{rope}, and \code{prob_in_rope}.
}
\description{
Posterior decision under Kruschke's (2018) Region of Practical Equivalence.
}
\examples{
draws <- rnorm(4000, mean = 0.05, sd = 0.1)
htdv_rope(draws, rope = c(-0.1, 0.1), level = 0.95)$decision

}
\references{
Kruschke, J.K. (2018). Rejecting or Accepting Parameter Values in Bayesian
Estimation. Advances in Methods and Practices in Psychological Science
1(2): 270-280.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{htdv_diagnostics}
\alias{htdv_diagnostics}
\title{MCMC Diagnostics for htdv_fit Objects}
\usage{
htdv_diagnostics(
  fit,
  rhat_threshold = 1.01,
  ess_threshold = 400,
  bfmi_threshold = 0.3
)
}
\arguments{
\item{fit}{An \code{htdv_fit} object.}

\item{rhat_threshold}{Convergence threshold for Rhat.}

\item{ess_threshold}{Minimum effective sample size.}

\item{bfmi_threshold}{Minimum acceptable energy-BFMI.}
}
\value{
A list with \code{rhat}, \code{ess_bulk}, \code{ess_tail},
  \code{bfmi}, \code{divergences}, \code{max_treedepth},
  \code{passed} (logical), and \code{failures} (character vector).
}
\description{
Computes split-Rhat, bulk and tail ESS, E-BFMI, divergence count, and
treedepth saturation; all standard HMC diagnostics (Vehtari et al. 2021,
Betancourt 2016).
}
\examples{
\donttest{
x <- rnorm(50)
fit <- htdv_fit(x, model = "tac", chains = 2, iter = 500,
                refresh = 0, seed = 1)
htdv_diagnostics(fit)$passed
}

}
\references{
Vehtari, A., Gelman, A., Simpson, D., Carpenter, B., & Buerkner, P.-C.
(2021). Rank-normalization, folding, and localization: An improved Rhat
for assessing convergence of MCMC. Bayesian Analysis 16(2): 667-718.
Betancourt, M. (2016). Diagnosing Suboptimal Cotangent Disintegrations.
<doi:10.48550/arXiv.1604.00695>.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decision.R
\name{htdv_stack}
\alias{htdv_stack}
\title{Predictive Stacking of Bayesian Models}
\usage{
htdv_stack(fits, method = c("log_score"))
}
\arguments{
\item{fits}{Named list of \code{htdv_fit} objects.}

\item{method}{Stacking objective. Only \code{"log_score"} supported in this
release.}
}
\value{
A list with \code{weights}, \code{model_names}, and
  \code{elpd_per_model}.
}
\description{
Computes stacking weights (Yao, Vehtari, Simpson, Gelman 2018) across a
list of fitted models by LOO-log-score maximization.
}
\examples{
\donttest{
if (requireNamespace("loo", quietly = TRUE)) {
  x <- rnorm(50)
  f1 <- htdv_fit(x, model = "tac", chains = 2, iter = 500,
                 refresh = 0, seed = 1)
  f2 <- htdv_fit(x, model = "wsc", chains = 2, iter = 500,
                 refresh = 0, seed = 2)
  htdv_stack(list(tac = f1, wsc = f2))$weights
}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conformal.R
\name{htdv_conformal}
\alias{htdv_conformal}
\title{Adaptive Conformal Inference for Dependent Data}
\usage{
htdv_conformal(
  x,
  predictor,
  residual_fun = function(e) abs(e),
  alpha_target = 0.1,
  lambda = 0.05,
  burn_in = 20L
)
}
\arguments{
\item{x}{Numeric vector of sequential observations.}

\item{predictor}{Function \code{function(history)} returning a scalar
one-step point prediction.}

\item{residual_fun}{Function of residuals to calibrate the score; default
absolute residual.}

\item{alpha_target}{Target miscoverage rate.}

\item{lambda}{Step size for the gradient update.}

\item{burn_in}{Integer; the first \code{burn_in} observations are used only
to warm up the predictor.}
}
\value{
A list with \code{intervals} (matrix of lower/upper),
  \code{coverage} (empirical), \code{alpha_path}, and
  \code{point_predictions}.
}
\description{
Online adaptive conformal prediction intervals with gradient update on the
miscoverage rate (Gibbs-Candes 2021). Provides long-run marginal coverage
without exchangeability.
}
\examples{
x <- arima.sim(model = list(ar = 0.6), n = 200, rand.gen = rnorm)
pred <- function(hist) if (length(hist) >= 1) hist[length(hist)] else 0
out <- htdv_conformal(as.numeric(x), pred, alpha_target = 0.1,
                      lambda = 0.05, burn_in = 20)
out$coverage

}
\references{
Gibbs, I., & Candes, E. (2021). Adaptive Conformal Inference under
Distribution Shift. NeurIPS 34: 1660-1672.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fixedb.R
\name{htdv_fixedb}
\alias{htdv_fixedb}
\title{Fixed-Bandwidth HAR Inference (Kiefer-Vogelsang 2005)}
\usage{
htdv_fixedb(
  x,
  theta0 = 0,
  B = 0.1,
  kernel = c("bartlett", "parzen", "qs"),
  sims = 5000L,
  alpha = 0.05
)
}
\arguments{
\item{x}{Numeric vector.}

\item{theta0}{Null value for the mean (default 0).}

\item{B}{Fixed bandwidth fraction in (0, 1].}

\item{kernel}{Kernel choice (see \code{\link{htdv_lrv}}).}

\item{sims}{Number of Monte Carlo draws for the critical-value simulation.}

\item{alpha}{Significance level.}
}
\value{
A list with \code{statistic}, \code{critical_value}, \code{p_value},
  \code{reject}, \code{bandwidth}, and \code{kernel}.
}
\description{
Fixed-b HAR Wald test where the bandwidth is a fixed fraction of the sample
size. Critical values are obtained by simulation from the asymptotic
functional distribution.
}
\examples{
set.seed(1)
x <- arima.sim(model = list(ar = 0.5), n = 200, rand.gen = rnorm)
htdv_fixedb(as.numeric(x), theta0 = 0, B = 0.2, sims = 1000)$p_value

}
\references{
Kiefer, N.M., & Vogelsang, T.J. (2005). A New Asymptotic Theory for
Heteroskedasticity-Autocorrelation Robust Tests. Econometric Theory
21(6): 1130-1164.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/long_run_var.R
\name{htdv_lrv}
\alias{htdv_lrv}
\title{Long-Run Variance Estimation (HAC)}
\usage{
htdv_lrv(
  x,
  kernel = c("bartlett", "parzen", "qs"),
  bandwidth = "andrews",
  prewhiten = FALSE
)
}
\arguments{
\item{x}{Numeric vector of observations.}

\item{kernel}{Kernel choice: \code{"bartlett"}, \code{"parzen"}, or
\code{"qs"} (Quadratic-Spectral).}

\item{bandwidth}{Either \code{"andrews"} (data-driven) or a positive
numeric scalar.}

\item{prewhiten}{Logical; if \code{TRUE}, AR(1) prewhiten-recolor
(Andrews-Monahan 1992).}
}
\value{
A list with components \code{lrv} (long-run variance estimate),
  \code{bandwidth}, \code{kernel}, and \code{ar_coef} (only if prewhitened).
}
\description{
Heteroskedasticity-and-autocorrelation-consistent long-run variance with
data-driven Andrews (1991) optimal bandwidth, Bartlett, Parzen, and
Quadratic-Spectral kernels.
}
\examples{
set_seed_user <- 42
x <- arima.sim(model = list(ar = 0.6), n = 200, rand.gen = rnorm)
out <- htdv_lrv(as.numeric(x), kernel = "qs", bandwidth = "andrews")
out$lrv

}
\references{
Andrews, D.W.K. (1991). Heteroskedasticity and Autocorrelation Consistent
Covariance Matrix Estimation. Econometrica 59(3): 817-858.
Newey, W.K. & West, K.D. (1987). A Simple, Positive Semi-Definite
Heteroskedasticity and Autocorrelation Consistent Covariance Matrix.
Econometrica 55(3): 703-708.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simstudy.R
\name{htdv_simstudy_summary}
\alias{htdv_simstudy_summary}
\title{Aggregate a Simulation-Study Output}
\usage{
htdv_simstudy_summary(res, alpha = 0.05)
}
\arguments{
\item{res}{Data frame returned by \code{htdv_simstudy}.}

\item{alpha}{Nominal size used in the study (must match what
\code{htdv_simstudy} was called with).}
}
\value{
A data frame with one row per (cell, layer) and columns
\code{n, phi, tail, imb, delta, layer, n_reps, reject_rate, coverage,
mean_ci_length, mean_runtime, diag_pass_rate}.
}
\description{
Computes empirical size (or rejection rate), coverage, and mean interval
length per (cell x layer) from the raw output of \code{\link{htdv_simstudy}}.
}
\examples{
\donttest{
res <- htdv_simstudy(n_grid = c(40), phi_grid = c(0.3),
                     tail_grid = "normal", imb_grid = 1,
                     delta_grid = c(0, 0.3), R = 5,
                     layers = c("har"))
htdv_simstudy_summary(res)
}

}
